@@ -6,7 +6,11 @@ API_HEADER="Accept: application/vnd.github.v3+json"
 AUTH_HEADER="Authorization: token $GITHUB_TOKEN"
 
 api_get() {
-  curl -X GET -s -H "${AUTH_HEADER}" -H "${API_HEADER}" "$1"
+  curl -X GET -s -H "${AUTH_HEADER}" -H "${API_HEADER}" "$1" > $RUNNER_TEMP/output.json
+  cat $RUNNER_TEMP/output.json 
+  if [[ "$RUNNER_DEBUG" == "1" ]]; then
+    cat $RUNNER_TEMP/output.json 1>&2
+  fi
 }
 
 api_post() {
