@@ -3,8 +3,6 @@
 source .github/bot-base.sh
 
 EXTENSION_REGEX='\.(cuh?|hpp|hpp\.inc?|cpp)$'
-FORMAT_HEADER_REGEX='^(benchmark|core|cuda|hip|include/ginkgo/core|omp|reference|dpcpp|common/unified|test)/'
-FORMAT_REGEX='^(common|examples)/'
 CLANG_FORMAT=clang-format-14
 
 echo -n "Collecting information on triggering PR"
