@@ -14,7 +14,6 @@ git config user.name "ginkgo-bot"
 
 # save scripts from develop
 cp .clang-format .pre-commit-config.yaml /tmp
-cp dev_tools/scripts/update_ginkgo_header.sh /tmp
 
 # checkout current PR head
 LOCAL_BRANCH=format-tmp-$HEAD_BRANCH
@@ -23,7 +22,6 @@ git checkout -b $LOCAL_BRANCH fork/$HEAD_BRANCH
 # restore files from develop
 cp /tmp/.clang-format .
 cp /tmp/.pre-commit-config.yaml .
-cp /tmp/update_ginkgo_header.sh dev_tools/scripts/
 
 # make base pre-commit config available
 git add .pre-commit-config.yaml
