@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -8,10 +8,8 @@
 
 #include <type_traits>
 
-
 #include <thrust/complex.h>
 
-
 #include "block_col_major.hpp"
 #include "reduced_row_major.hpp"
 #include "row_major.hpp"
@@ -19,7 +17,19 @@
 #include "utils.hpp"
 
 
+struct __half;
+
+struct __nv_bfloat16;
+
+
 namespace gko {
+
+
+class half;
+
+class bfloat16;
+
+
 namespace acc {
 namespace detail {
 
@@ -29,6 +39,16 @@ struct cuda_type {
     using type = T;
 };
 
+template <>
+struct cuda_type<gko::half> {
+    using type = __half;
+};
+
+template <>
+struct cuda_type<gko::bfloat16> {
+    using type = __nv_bfloat16;
+};
+
 // Unpack cv and reference / pointer qualifiers
 template <typename T>
 struct cuda_type<const T> {
@@ -59,7 +79,7 @@ struct cuda_type<T&&> {
 // Transform std::complex to thrust::complex
 template <typename T>
 struct cuda_type<std::complex<T>> {
-    using type = thrust::complex<T>;
+    using type = thrust::complex<typename cuda_type<T>::type>;
 };
 
 
