@@ -1,10 +1,7 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/ginkgo.hpp>
-
-
 #include <algorithm>
 #include <chrono>
 #include <cstdlib>
@@ -14,6 +11,7 @@
 #include <iostream>
 #include <typeinfo>
 
+#include <ginkgo/ginkgo.hpp>
 
 #include "benchmark/utils/formats.hpp"
 #include "benchmark/utils/general.hpp"
@@ -88,8 +86,7 @@ struct ConversionBenchmark : Benchmark<gko::device_matrix_data<etype, itype>> {
     gko::device_matrix_data<etype, itype> setup(
         std::shared_ptr<gko::Executor> exec, json& test_case) const override
     {
-        gko::matrix_data<etype, itype> data;
-        data = Generator::generate_matrix_data(test_case);
+        auto [data, local_size] = Generator::generate_matrix_data(test_case);
         // no reordering here, as it doesn't impact conversions beyond
         // dense-sparse conversions
         std::clog << "Matrix is of size (" << data.size[0] << ", "
@@ -165,9 +162,9 @@ int main(int argc, char* argv[])
 
     std::string extra_information =
         std::string() + "The formats are " + FLAGS_formats;
-    print_general_information(extra_information);
 
     auto exec = executor_factory.at(FLAGS_executor)(FLAGS_gpu_timer);
+    print_general_information(extra_information, exec);
     auto formats = split(FLAGS_formats, ',');
 
     auto test_cases = json::parse(get_input_stream());
