@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -38,9 +38,10 @@ DEFINE_string(solvers, "cg",
               "A comma-separated list of solvers to run. "
               "Supported values are: bicgstab, bicg, cb_gmres_keep, "
               "cb_gmres_reduce1, cb_gmres_reduce2, cb_gmres_integer, "
-              "cb_gmres_ireduce1, cb_gmres_ireduce2, cg, cgs, fcg, gmres, idr, "
-              "lower_trs, upper_trs, spd_direct, symm_direct, "
-              "near_symm_direct, direct, overhead");
+              "cb_gmres_ireduce1, cb_gmres_ireduce2, cg, cgs, direct, fcg, "
+              "pipe_cg, gmres, idr, lower_trs, minres, near_symm_direct, "
+              "upper_trs, spd_direct, symm_direct, "
+              "overhead");
 
 DEFINE_uint32(
     nrhs, 1,
@@ -189,6 +190,9 @@ std::unique_ptr<gko::LinOpFactory> generate_solver(
     } else if (description == "fcg") {
         return add_criteria_precond_finalize<gko::solver::Fcg<etype>>(
             exec, precond, max_iters);
+    } else if (description == "pipe_cg") {
+        return add_criteria_precond_finalize<gko::solver::PipeCg<etype>>(
+            exec, precond, max_iters);
     } else if (description == "idr") {
         return add_criteria_precond_finalize(
             gko::solver::Idr<etype>::build()
@@ -200,6 +204,9 @@ std::unique_ptr<gko::LinOpFactory> generate_solver(
             gko::solver::Gmres<etype>::build().with_krylov_dim(
                 FLAGS_gmres_restart),
             exec, precond, max_iters);
+    } else if (description == "minres") {
+        return add_criteria_precond_finalize<gko::solver::Minres<etype>>(
+            exec, precond, max_iters);
     } else if (description == "lower_trs") {
         return gko::solver::LowerTrs<etype>::build()
             .with_num_rhs(FLAGS_nrhs)
@@ -291,28 +298,19 @@ struct SolverGenerator : DefaultSystemGenerator<> {
             return gko::read_generic<Vec>(rhs_fd, std::move(exec));
         } else {
             gko::dim<2> vec_size{system_matrix->get_size()[0], FLAGS_nrhs};
+            gko::dim<2> local_vec_size{
+                gko::detail::get_local(system_matrix)->get_size()[1],
+                FLAGS_nrhs};
             if (FLAGS_rhs_generation == "1") {
-                return create_multi_vector(exec, vec_size, gko::one<etype>());
+                return create_multi_vector(exec, vec_size, local_vec_size,
+                                           gko::one<etype>());
             } else if (FLAGS_rhs_generation == "random") {
-                return create_multi_vector_random(exec, vec_size);
+                return create_multi_vector_random(exec, vec_size,
+                                                  local_vec_size);
             } else if (FLAGS_rhs_generation == "sinus") {
-                auto rhs = vec<etype>::create(exec, vec_size);
-
-                auto tmp = create_matrix_sin<etype>(exec, vec_size);
-                auto scalar = gko::matrix::Dense<rc_etype>::create(
-                    exec->get_master(), gko::dim<2>{1, vec_size[1]});
-                tmp->compute_norm2(scalar);
-                for (gko::size_type i = 0; i < vec_size[1]; ++i) {
-                    scalar->at(0, i) = gko::one<rc_etype>() / scalar->at(0, i);
-                }
-                // normalize sin-vector
-                if (gko::is_complex_s<etype>::value) {
-                    tmp->scale(scalar->make_complex());
-                } else {
-                    tmp->scale(scalar);
-                }
-                system_matrix->apply(tmp, rhs);
-                return rhs;
+                return create_normalized_manufactured_rhs(
+                    exec, system_matrix,
+                    create_matrix_sin<etype>(exec, vec_size).get());
             }
             throw std::invalid_argument(std::string("\"rhs_generation\" = ") +
                                         FLAGS_rhs_generation +
@@ -325,10 +323,13 @@ struct SolverGenerator : DefaultSystemGenerator<> {
         const gko::LinOp* system_matrix, const Vec* rhs) const
     {
         gko::dim<2> vec_size{system_matrix->get_size()[1], FLAGS_nrhs};
+        gko::dim<2> local_vec_size{
+            gko::detail::get_local(system_matrix)->get_size()[1], FLAGS_nrhs};
         if (FLAGS_initial_guess_generation == "0") {
-            return create_multi_vector(exec, vec_size, gko::zero<etype>());
+            return create_multi_vector(exec, vec_size, local_vec_size,
+                                       gko::zero<etype>());
         } else if (FLAGS_initial_guess_generation == "random") {
-            return create_multi_vector_random(exec, vec_size);
+            return create_multi_vector_random(exec, vec_size, local_vec_size);
         } else if (FLAGS_initial_guess_generation == "rhs") {
             return rhs->clone();
         }
@@ -414,11 +415,12 @@ struct SolverBenchmark : Benchmark<solver_benchmark_state<Generator>> {
                 {std::numeric_limits<rc_etype>::quiet_NaN()}, exec);
             state.x = generator.initialize({0.0}, exec);
         } else {
-            auto data = generator.generate_matrix_data(test_case);
+            auto [data, size] = generator.generate_matrix_data(test_case);
             auto permutation = reorder(data, test_case);
 
             state.system_matrix = generator.generate_matrix_with_format(
-                exec, test_case["optimal"]["spmv"].get<std::string>(), data);
+                exec, test_case["optimal"]["spmv"].get<std::string>(), data,
+                size);
             state.b = generator.generate_rhs(exec, state.system_matrix.get(),
                                              test_case);
             if (permutation) {
