@@ -2,12 +2,10 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/ginkgo.hpp>
-
-
 #include <cstdlib>
 #include <iostream>
 
+#include <ginkgo/ginkgo.hpp>
 
 #include "benchmark/spmv/spmv_common.hpp"
 #include "benchmark/utils/formats.hpp"
@@ -28,10 +26,11 @@ int main(int argc, char* argv[])
     std::string extra_information = "The formats are " + FLAGS_formats +
                                     "\nThe number of right hand sides is " +
                                     std::to_string(FLAGS_nrhs);
-    print_general_information(extra_information);
 
     auto exec = executor_factory.at(FLAGS_executor)(FLAGS_gpu_timer);
 
+    print_general_information(extra_information, exec);
+
     auto test_cases = json::parse(get_input_stream());
 
     run_test_cases(SpmvBenchmark<Generator>{Generator{}, split(FLAGS_formats)},
