@@ -40,3 +40,12 @@ test_framework.compare_output_distributed(
     expected_stderr="spmv_distributed.profile.stderr",
     num_procs=3,
 )
+
+# complex
+test_framework.compare_output_distributed(
+    ["-input", '[{"size": 100, "stencil": "7pt", "comm_pattern": "stencil"}]'],
+    expected_stdout="spmv_distributed_dcomplex.simple.stdout",
+    expected_stderr="spmv_distributed_dcomplex.simple.stderr",
+    num_procs=3,
+    use_complex=True
+)
