@@ -2,14 +2,11 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/ginkgo.hpp>
-
-
 #include <memory>
 
-
 #include <oneapi/mkl.hpp>
 
+#include <ginkgo/ginkgo.hpp>
 
 #include "benchmark/utils/sparselib_linops.hpp"
 #include "benchmark/utils/types.hpp"
@@ -50,12 +47,16 @@ protected:
         GKO_NOT_IMPLEMENTED;
     }
 
-    void initialize_mat_handle()
+    void initialize_mat_handle(std::shared_ptr<const gko::DpcppExecutor> exec)
     {
         mat_handle_ = handle_manager<oneapi::mkl::sparse::matrix_handle>(
             create_mat_handle(),
-            [](oneapi::mkl::sparse::matrix_handle_t mat_handle) {
-                oneapi::mkl::sparse::release_matrix_handle(&mat_handle);
+            [exec](oneapi::mkl::sparse::matrix_handle_t mat_handle) {
+                oneapi::mkl::sparse::release_matrix_handle(
+#if INTEL_MKL_VERSION >= 20240000
+                    *exec->get_queue(),
+#endif
+                    &mat_handle);
             });
     }
 
@@ -66,7 +67,7 @@ protected:
         if (this->get_device_exec() == nullptr) {
             GKO_NOT_IMPLEMENTED;
         }
-        this->initialize_mat_handle();
+        this->initialize_mat_handle(this->get_device_exec());
     }
 
     ~OnemklBase() = default;
@@ -77,7 +78,7 @@ protected:
     {
         if (this != &other) {
             gko::LinOp::operator=(other);
-            this->initialize_mat_handle();
+            this->initialize_mat_handle(this->get_device_exec());
         }
         return *this;
     }
@@ -109,6 +110,9 @@ public:
         this->set_size(csr_->get_size());
 
         oneapi::mkl::sparse::set_csr_data(
+#if INTEL_MKL_VERSION >= 20240000
+            *(this->get_device_exec()->get_queue()),
+#endif
             this->get_mat_handle(), static_cast<int>(this->get_size()[0]),
             static_cast<int>(this->get_size()[1]),
             oneapi::mkl::index_base::zero, csr_->get_row_ptrs(),
