@@ -1,9 +1,8 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/base/mtx_io.hpp>
-
+#include "ginkgo/core/base/mtx_io.hpp"
 
 #include <algorithm>
 #include <cctype>
@@ -14,8 +13,9 @@
 #include <string>
 #include <type_traits>
 
-
+#include <ginkgo/core/base/bfloat16.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
+#include <ginkgo/core/base/half.hpp>
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/utils.hpp>
 
@@ -37,6 +37,9 @@ namespace {
     }
 
 
+constexpr auto max_streamsize = std::numeric_limits<std::streamsize>::max();
+
+
 /**
  * The mtx_io class provides the functionality of reading and writing matrix
  * market format files.
@@ -516,6 +519,8 @@ private:
                 GKO_CHECK_STREAM(content, "error when reading matrix entry " +
                                               std::to_string(i));
                 modifier->insert_entry(row - 1, col - 1, entry, data);
+                // discards rest of the line
+                content.ignore(max_streamsize, '\n');
             }
             return data;
         }
@@ -571,7 +576,7 @@ private:
             size_type num_cols{};
             GKO_CHECK_STREAM(
                 header >> num_rows >> num_cols,
-                "error when determining matrix size, expected: rows cols nnz");
+                "error when determining matrix size, expected: rows cols");
             matrix_data<ValueType, IndexType> data(dim<2>{num_rows, num_cols});
             data.nonzeros.reserve(modifier->get_reservation_size(
                 num_rows, num_cols, num_rows * num_cols));
@@ -584,6 +589,8 @@ private:
                                          std::to_string(row) + " ," +
                                          std::to_string(col));
                     modifier->insert_entry(row, col, entry, data);
+                    // discards rest of the line
+                    content.ignore(max_streamsize, '\n');
                 }
             }
             return data;
@@ -752,19 +759,37 @@ static constexpr uint64 binary_format_magic()
 {
     constexpr auto is_int = std::is_same<IndexType, int32>::value;
     constexpr auto is_long = std::is_same<IndexType, int64>::value;
+    constexpr auto is_half = std::is_same<ValueType, half>::value;
+    constexpr auto is_bf16 = std::is_same<ValueType, bfloat16>::value;
     constexpr auto is_double = std::is_same<ValueType, double>::value;
     constexpr auto is_float = std::is_same<ValueType, float>::value;
     constexpr auto is_complex_double =
         std::is_same<ValueType, std::complex<double>>::value;
     constexpr auto is_complex_float =
         std::is_same<ValueType, std::complex<float>>::value;
+    constexpr auto is_complex_half =
+        std::is_same<ValueType, std::complex<half>>::value;
+    constexpr auto is_complex_bf16 =
+        std::is_same<ValueType, std::complex<bfloat16>>::value;
     static_assert(is_int || is_long, "invalid storage index type");
-    static_assert(
-        is_double || is_float || is_complex_double || is_complex_float,
-        "invalid storage value type");
+    static_assert(is_bf16 || is_complex_bf16 || is_half || is_complex_half ||
+                      is_double || is_float || is_complex_double ||
+                      is_complex_float,
+                  "invalid storage value type");
     constexpr auto index_bit = is_int ? 'I' : 'L';
     constexpr auto value_bit =
-        is_double ? 'D' : (is_float ? 'S' : (is_complex_double ? 'Z' : 'C'));
+        is_double
+            ? 'D'
+            : (is_float
+                   ? 'S'
+                   : (is_complex_double
+                          ? 'Z'
+                          : (is_complex_float
+                                 ? 'C'
+                                 : (is_half ? 'H'
+                                            : (is_complex_half
+                                                   ? 'X'
+                                                   : (is_bf16 ? 'B' : 'Y'))))));
     constexpr uint64 shift = 256;
     constexpr uint64 type_bits = index_bit * shift + value_bit;
     return 'G' +
@@ -874,12 +899,20 @@ matrix_data<ValueType, IndexType> read_binary_raw(std::istream& is)
     }
     DECLARE_OVERLOAD(double, int32)
     DECLARE_OVERLOAD(float, int32)
+    DECLARE_OVERLOAD(float16, int32)
+    DECLARE_OVERLOAD(bfloat16, int32)
     DECLARE_OVERLOAD(std::complex<double>, int32)
     DECLARE_OVERLOAD(std::complex<float>, int32)
+    DECLARE_OVERLOAD(std::complex<float16>, int32)
+    DECLARE_OVERLOAD(std::complex<bfloat16>, int32)
     DECLARE_OVERLOAD(double, int64)
     DECLARE_OVERLOAD(float, int64)
+    DECLARE_OVERLOAD(float16, int64)
+    DECLARE_OVERLOAD(bfloat16, int64)
     DECLARE_OVERLOAD(std::complex<double>, int64)
     DECLARE_OVERLOAD(std::complex<float>, int64)
+    DECLARE_OVERLOAD(std::complex<float16>, int64)
+    DECLARE_OVERLOAD(std::complex<bfloat16>, int64)
 #undef DECLARE_OVERLOAD
     else
     {
