@@ -2,18 +2,15 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/base/timer.hpp>
-
+#include "ginkgo/core/base/timer.hpp"
 
 #include <chrono>
 #include <memory>
 #include <utility>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 
-
 #include "cuda/base/device.hpp"
 #include "dpcpp/base/device.hpp"
 #include "hip/base/device.hpp"
