@@ -1,15 +1,20 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/distributed/matrix.hpp>
+#include "ginkgo/core/distributed/matrix.hpp"
 
+#include <utility>
 
+#include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/precision_dispatch.hpp>
+#include <ginkgo/core/distributed/assembly.hpp>
+#include <ginkgo/core/distributed/neighborhood_communicator.hpp>
+#include <ginkgo/core/distributed/partition_helpers.hpp>
 #include <ginkgo/core/distributed/vector.hpp>
 #include <ginkgo/core/matrix/coo.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
-
+#include <ginkgo/core/matrix/diagonal.hpp>
 
 #include "core/distributed/matrix_kernels.hpp"
 
@@ -43,16 +48,11 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(
     std::shared_ptr<const Executor> exec, mpi::communicator comm,
     ptr_param<const LinOp> local_matrix_template,
     ptr_param<const LinOp> non_local_matrix_template)
-    : EnableDistributedLinOp<
-          Matrix<value_type, local_index_type, global_index_type>>{exec},
+    : EnableLinOp<Matrix>{exec},
       DistributedBase{comm},
-      send_offsets_(comm.size() + 1),
-      send_sizes_(comm.size()),
-      recv_offsets_(comm.size() + 1),
-      recv_sizes_(comm.size()),
-      gather_idxs_{exec},
-      non_local_to_global_{exec},
-      one_scalar_{},
+      row_gatherer_{RowGatherer<LocalIndexType>::create(exec, comm)},
+      imap_{exec},
+      one_scalar_{exec, 1.0},
       local_mtx_{local_matrix_template->clone(exec)},
       non_local_mtx_{non_local_matrix_template->clone(exec)}
 {
@@ -62,84 +62,44 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(
     GKO_ASSERT(
         (dynamic_cast<ReadableFromMatrixData<ValueType, LocalIndexType>*>(
             non_local_mtx_.get())));
-    one_scalar_.init(exec, dim<2>{1, 1});
-    one_scalar_->fill(one<value_type>());
 }
 
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(
     std::shared_ptr<const Executor> exec, mpi::communicator comm, dim<2> size,
     std::shared_ptr<LinOp> local_linop)
-    : EnableDistributedLinOp<
-          Matrix<value_type, local_index_type, global_index_type>>{exec},
+    : EnableLinOp<Matrix>{exec},
       DistributedBase{comm},
-      send_offsets_(comm.size() + 1),
-      send_sizes_(comm.size()),
-      recv_offsets_(comm.size() + 1),
-      recv_sizes_(comm.size()),
-      gather_idxs_{exec},
-      non_local_to_global_{exec},
-      one_scalar_{},
+      row_gatherer_{RowGatherer<LocalIndexType>::create(exec, comm)},
+      imap_{exec},
+      one_scalar_{exec, 1.0},
       non_local_mtx_(::gko::matrix::Coo<ValueType, LocalIndexType>::create(
           exec, dim<2>{local_linop->get_size()[0], 0}))
 {
     this->set_size(size);
-    one_scalar_.init(exec, dim<2>{1, 1});
-    one_scalar_->fill(one<value_type>());
     local_mtx_ = std::move(local_linop);
 }
 
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(
-    std::shared_ptr<const Executor> exec, mpi::communicator comm, dim<2> size,
-    std::shared_ptr<LinOp> local_linop, std::shared_ptr<LinOp> non_local_linop,
-    std::vector<comm_index_type> recv_sizes,
-    std::vector<comm_index_type> recv_offsets,
-    array<local_index_type> recv_gather_idxs)
-    : EnableDistributedLinOp<
-          Matrix<value_type, local_index_type, global_index_type>>{exec},
+    std::shared_ptr<const Executor> exec, mpi::communicator comm,
+    index_map<LocalIndexType, GlobalIndexType> imap,
+    std::shared_ptr<LinOp> local_linop, std::shared_ptr<LinOp> non_local_linop)
+    : EnableLinOp<Matrix>{exec},
       DistributedBase{comm},
-      send_offsets_(comm.size() + 1),
-      send_sizes_(comm.size()),
-      recv_offsets_(comm.size() + 1),
-      recv_sizes_(comm.size()),
-      gather_idxs_{exec},
-      non_local_to_global_{exec},
-      one_scalar_{}
+      row_gatherer_(RowGatherer<LocalIndexType>::create(exec, comm)),
+      imap_(std::move(imap)),
+      one_scalar_{exec, 1.0}
 {
-    this->set_size(size);
+    this->set_size({imap_.get_global_size(), imap_.get_global_size()});
     local_mtx_ = std::move(local_linop);
     non_local_mtx_ = std::move(non_local_linop);
-    recv_offsets_ = std::move(recv_offsets);
-    recv_sizes_ = std::move(recv_sizes);
-    // build send information from recv copy
-    // exchange step 1: determine recv_sizes, send_sizes, send_offsets
-    std::partial_sum(recv_sizes_.begin(), recv_sizes_.end(),
-                     recv_offsets_.begin() + 1);
-    comm.all_to_all(exec, recv_sizes_.data(), 1, send_sizes_.data(), 1);
-    std::partial_sum(send_sizes_.begin(), send_sizes_.end(),
-                     send_offsets_.begin() + 1);
-    send_offsets_[0] = 0;
-    recv_offsets_[0] = 0;
 
-    // exchange step 2: exchange gather_idxs from receivers to senders
-    auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
-    if (use_host_buffer) {
-        recv_gather_idxs.set_executor(exec->get_master());
-        gather_idxs_.clear();
-        gather_idxs_.set_executor(exec->get_master());
-    }
-    gather_idxs_.resize_and_reset(send_offsets_.back());
-    comm.all_to_all_v(use_host_buffer ? exec->get_master() : exec,
-                      recv_gather_idxs.get_const_data(), recv_sizes_.data(),
-                      recv_offsets_.data(), gather_idxs_.get_data(),
-                      send_sizes_.data(), send_offsets_.data());
-    if (use_host_buffer) {
-        gather_idxs_.set_executor(exec);
-    }
-
-    one_scalar_.init(exec, dim<2>{1, 1});
-    one_scalar_->fill(one<value_type>());
+    row_gatherer_ = RowGatherer<LocalIndexType>::create(
+        row_gatherer_->get_executor(),
+        row_gatherer_->get_collective_communicator()->create_with_same_type(
+            comm, &imap_),
+        imap_);
 }
 
 
@@ -193,9 +153,46 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::create(
     std::vector<comm_index_type> recv_offsets,
     array<local_index_type> recv_gather_idxs)
 {
-    return std::unique_ptr<Matrix>{new Matrix{exec, comm, size, local_linop,
-                                              non_local_linop, recv_sizes,
-                                              recv_offsets, recv_gather_idxs}};
+    array<comm_index_type> part_ids(exec->get_master(), comm.size());
+    std::iota(part_ids.get_data(), part_ids.get_data() + part_ids.get_size(),
+              0);
+    auto contiguous_partition =
+        share(build_partition_from_local_size<LocalIndexType, GlobalIndexType>(
+            exec, comm, local_linop->get_size()[0]));
+    array<global_index_type> global_recv_gather_idxs(
+        exec, recv_gather_idxs.get_size());
+    for (int rank = 0; rank < comm.size(); ++rank) {
+        if (recv_sizes[rank] > 0) {
+            auto map = index_map<LocalIndexType, GlobalIndexType>(
+                exec, contiguous_partition, rank, array<GlobalIndexType>{exec});
+            auto local_view = make_array_view(
+                exec, recv_sizes[rank],
+                recv_gather_idxs.get_data() + recv_offsets[rank]);
+            auto global_idxs =
+                map.map_to_global(local_view, index_space::local);
+            exec->copy(recv_sizes[rank], global_idxs.get_const_data(),
+                       global_recv_gather_idxs.get_data() + recv_offsets[rank]);
+        }
+    }
+
+    return Matrix::create(
+        exec, comm,
+        index_map<LocalIndexType, GlobalIndexType>(
+            exec, contiguous_partition, comm.rank(), global_recv_gather_idxs),
+        std::move(local_linop), std::move(non_local_linop));
+}
+
+
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+std::unique_ptr<Matrix<ValueType, LocalIndexType, GlobalIndexType>>
+Matrix<ValueType, LocalIndexType, GlobalIndexType>::create(
+    std::shared_ptr<const Executor> exec, mpi::communicator comm,
+    index_map<LocalIndexType, GlobalIndexType> imap,
+    std::shared_ptr<LinOp> local_linop, std::shared_ptr<LinOp> non_local_linop)
+{
+    return std::unique_ptr<Matrix>{
+        new Matrix{std::move(exec), comm, std::move(imap),
+                   std::move(local_linop), std::move(non_local_linop)}};
 }
 
 
@@ -208,12 +205,8 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::convert_to(
                result->get_communicator().size());
     result->local_mtx_->copy_from(this->local_mtx_);
     result->non_local_mtx_->copy_from(this->non_local_mtx_);
-    result->gather_idxs_ = this->gather_idxs_;
-    result->send_offsets_ = this->send_offsets_;
-    result->recv_offsets_ = this->recv_offsets_;
-    result->recv_sizes_ = this->recv_sizes_;
-    result->send_sizes_ = this->send_sizes_;
-    result->non_local_to_global_ = this->non_local_to_global_;
+    result->row_gatherer_->copy_from(this->row_gatherer_);
+    result->imap_ = this->imap_;
     result->set_size(this->get_size());
 }
 
@@ -227,24 +220,87 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::move_to(
                result->get_communicator().size());
     result->local_mtx_->move_from(this->local_mtx_);
     result->non_local_mtx_->move_from(this->non_local_mtx_);
-    result->gather_idxs_ = std::move(this->gather_idxs_);
-    result->send_offsets_ = std::move(this->send_offsets_);
-    result->recv_offsets_ = std::move(this->recv_offsets_);
-    result->recv_sizes_ = std::move(this->recv_sizes_);
-    result->send_sizes_ = std::move(this->send_sizes_);
-    result->non_local_to_global_ = std::move(this->non_local_to_global_);
+    result->row_gatherer_->move_from(this->row_gatherer_);
+    result->imap_ = std::move(this->imap_);
     result->set_size(this->get_size());
     this->set_size({});
 }
 
 
+#if GINKGO_ENABLE_HALF || GINKGO_ENABLE_BFLOAT16
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::convert_to(
+    Matrix<next_precision<value_type, 2>, local_index_type, global_index_type>*
+        result) const
+{
+    GKO_ASSERT(this->get_communicator().size() ==
+               result->get_communicator().size());
+    result->local_mtx_->copy_from(this->local_mtx_.get());
+    result->non_local_mtx_->copy_from(this->non_local_mtx_.get());
+    result->row_gatherer_->copy_from(this->row_gatherer_);
+    result->imap_ = this->imap_;
+    result->set_size(this->get_size());
+}
+
+
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::move_to(
+    Matrix<next_precision<value_type, 2>, local_index_type, global_index_type>*
+        result)
+{
+    GKO_ASSERT(this->get_communicator().size() ==
+               result->get_communicator().size());
+    result->local_mtx_->move_from(this->local_mtx_.get());
+    result->non_local_mtx_->move_from(this->non_local_mtx_.get());
+    result->row_gatherer_->move_from(this->row_gatherer_);
+    result->imap_ = std::move(this->imap_);
+    result->set_size(this->get_size());
+    this->set_size({});
+}
+#endif
+
+
+#if GINKGO_ENABLE_HALF && GINKGO_ENABLE_BFLOAT16
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::convert_to(
+    Matrix<next_precision<value_type, 3>, local_index_type, global_index_type>*
+        result) const
+{
+    GKO_ASSERT(this->get_communicator().size() ==
+               result->get_communicator().size());
+    result->local_mtx_->copy_from(this->local_mtx_.get());
+    result->non_local_mtx_->copy_from(this->non_local_mtx_.get());
+    result->row_gatherer_->copy_from(this->row_gatherer_);
+    result->imap_ = this->imap_;
+    result->set_size(this->get_size());
+}
+
+
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::move_to(
+    Matrix<next_precision<value_type, 3>, local_index_type, global_index_type>*
+        result)
+{
+    GKO_ASSERT(this->get_communicator().size() ==
+               result->get_communicator().size());
+    result->local_mtx_->move_from(this->local_mtx_.get());
+    result->non_local_mtx_->move_from(this->non_local_mtx_.get());
+    result->row_gatherer_->move_from(this->row_gatherer_);
+    result->imap_ = std::move(this->imap_);
+    result->set_size(this->get_size());
+    this->set_size({});
+}
+#endif
+
+
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     const device_matrix_data<value_type, global_index_type>& data,
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
         row_partition,
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
-        col_partition)
+        col_partition,
+    assembly_mode assembly_type)
 {
     const auto comm = this->get_communicator();
     GKO_ASSERT_EQ(data.get_size()[0], row_partition->get_size());
@@ -253,9 +309,21 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     GKO_ASSERT_EQ(comm.size(), col_partition->get_num_parts());
     auto exec = this->get_executor();
     auto local_part = comm.rank();
+    auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
+    auto tmp_row_partition = make_temporary_clone(exec, row_partition);
+    auto tmp_col_partition = make_temporary_clone(exec, col_partition);
+
+    const device_matrix_data<value_type, global_index_type>* all_data_ptr =
+        &data;
+    device_matrix_data<value_type, global_index_type> assembled_data(exec);
+    if (assembly_type == assembly_mode::communicate) {
+        assembled_data = assemble_rows_from_neighbors<ValueType, LocalIndexType,
+                                                      GlobalIndexType>(
+            this->get_communicator(), data, row_partition);
+        all_data_ptr = &assembled_data;
+    }
 
     // set up LinOp sizes
-    auto num_parts = static_cast<size_type>(row_partition->get_num_parts());
     auto global_num_rows = row_partition->get_size();
     auto global_num_cols = col_partition->get_size();
     dim<2> global_dim{global_num_rows, global_num_cols};
@@ -274,21 +342,15 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     // as well as the rows of the non-local block. The columns of the non-local
     // block are still in global indices.
     exec->run(matrix::make_separate_local_nonlocal(
-        data, make_temporary_clone(exec, row_partition).get(),
-        make_temporary_clone(exec, col_partition).get(), local_part,
-        local_row_idxs, local_col_idxs, local_values, non_local_row_idxs,
-        global_non_local_col_idxs, non_local_values));
+        *all_data_ptr, tmp_row_partition.get(), tmp_col_partition.get(),
+        local_part, local_row_idxs, local_col_idxs, local_values,
+        non_local_row_idxs, global_non_local_col_idxs, non_local_values));
 
-    auto imap = index_map<local_index_type, global_index_type>(
+    imap_ = index_map<local_index_type, global_index_type>(
         exec, col_partition, comm.rank(), global_non_local_col_idxs);
 
     auto non_local_col_idxs =
-        imap.map_to_local(global_non_local_col_idxs, index_space::non_local);
-    non_local_to_global_ =
-        make_const_array_view(
-            imap.get_executor(), imap.get_remote_global_idxs().get_size(),
-            imap.get_remote_global_idxs().get_const_flat_data())
-            .copy_to_array();
+        imap_.map_to_local(global_non_local_col_idxs, index_space::non_local);
 
     // read the local matrix data
     const auto num_local_rows =
@@ -299,7 +361,7 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
         exec, dim<2>{num_local_rows, num_local_cols}, std::move(local_row_idxs),
         std::move(local_col_idxs), std::move(local_values)};
     device_matrix_data<value_type, local_index_type> non_local_data{
-        exec, dim<2>{num_local_rows, imap.get_remote_global_idxs().get_size()},
+        exec, dim<2>{num_local_rows, imap_.get_remote_global_idxs().get_size()},
         std::move(non_local_row_idxs), std::move(non_local_col_idxs),
         std::move(non_local_values)};
     as<ReadableFromMatrixData<ValueType, LocalIndexType>>(this->local_mtx_)
@@ -307,49 +369,11 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     as<ReadableFromMatrixData<ValueType, LocalIndexType>>(this->non_local_mtx_)
         ->read(std::move(non_local_data));
 
-    // exchange step 1: determine recv_sizes, send_sizes, send_offsets
-    auto host_recv_targets =
-        make_temporary_clone(exec->get_master(), &imap.get_remote_target_ids());
-    auto host_offsets = make_temporary_clone(
-        exec->get_master(), &imap.get_remote_global_idxs().get_offsets());
-    auto compute_recv_sizes = [](const auto* recv_targets, size_type size,
-                                 const auto* offsets, auto& recv_sizes) {
-        for (size_type i = 0; i < size; ++i) {
-            recv_sizes[recv_targets[i]] = offsets[i + 1] - offsets[i];
-        }
-    };
-    std::fill(recv_sizes_.begin(), recv_sizes_.end(), 0);
-    compute_recv_sizes(host_recv_targets->get_const_data(),
-                       host_recv_targets->get_size(),
-                       host_offsets->get_const_data(), recv_sizes_);
-    std::partial_sum(recv_sizes_.begin(), recv_sizes_.end(),
-                     recv_offsets_.begin() + 1);
-    comm.all_to_all(exec, recv_sizes_.data(), 1, send_sizes_.data(), 1);
-    std::partial_sum(send_sizes_.begin(), send_sizes_.end(),
-                     send_offsets_.begin() + 1);
-    send_offsets_[0] = 0;
-    recv_offsets_[0] = 0;
-
-    // exchange step 2: exchange gather_idxs from receivers to senders
-    auto recv_gather_idxs =
-        make_const_array_view(
-            imap.get_executor(), imap.get_non_local_size(),
-            imap.get_remote_local_idxs().get_const_flat_data())
-            .copy_to_array();
-    auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
-    if (use_host_buffer) {
-        recv_gather_idxs.set_executor(exec->get_master());
-        gather_idxs_.clear();
-        gather_idxs_.set_executor(exec->get_master());
-    }
-    gather_idxs_.resize_and_reset(send_offsets_.back());
-    comm.all_to_all_v(use_host_buffer ? exec->get_master() : exec,
-                      recv_gather_idxs.get_const_data(), recv_sizes_.data(),
-                      recv_offsets_.data(), gather_idxs_.get_data(),
-                      send_sizes_.data(), send_offsets_.data());
-    if (use_host_buffer) {
-        gather_idxs_.set_executor(exec);
-    }
+    row_gatherer_ = RowGatherer<LocalIndexType>::create(
+        row_gatherer_->get_executor(),
+        row_gatherer_->get_collective_communicator()->create_with_same_type(
+            comm, &imap_),
+        imap_);
 }
 
 
@@ -359,12 +383,13 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
         row_partition,
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
-        col_partition)
+        col_partition,
+    assembly_mode assembly_type)
 {
     return this->read_distributed(
         device_matrix_data<value_type, global_index_type>::create_from_host(
             this->get_executor(), data),
-        row_partition, col_partition);
+        row_partition, col_partition, assembly_type);
 }
 
 
@@ -372,71 +397,49 @@ template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
     const matrix_data<ValueType, global_index_type>& data,
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
-        partition)
+        partition,
+    assembly_mode assembly_type)
 {
     return this->read_distributed(
         device_matrix_data<value_type, global_index_type>::create_from_host(
             this->get_executor(), data),
-        partition, partition);
+        partition, partition, assembly_type);
 }
 
 
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 void Matrix<ValueType, LocalIndexType, GlobalIndexType>::read_distributed(
-    const device_matrix_data<ValueType, GlobalIndexType>& data,
+    const device_matrix_data<value_type, global_index_type>& data,
     std::shared_ptr<const Partition<local_index_type, global_index_type>>
-        partition)
+        partition,
+    assembly_mode assembly_type)
 {
-    return this->read_distributed(data, partition, partition);
+    return this->read_distributed(data, partition, partition, assembly_type);
 }
 
 
-template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
-mpi::request Matrix<ValueType, LocalIndexType, GlobalIndexType>::communicate(
-    const local_vector_type* local_b) const
+template <typename ValueType, typename LocalIndexType>
+std::pair<std::shared_ptr<Vector<ValueType>>,
+          std::shared_ptr<Vector<ValueType>>>
+init_recv_buffers(std::shared_ptr<const Executor> exec,
+                  const RowGatherer<LocalIndexType>* row_gatherer,
+                  size_type num_cols, const detail::GenericVectorCache& buffer,
+                  const detail::GenericVectorCache& host_buffer)
 {
-    // This function can never return early!
-    // Even if the non-local part is empty, i.e. this process doesn't need
-    // any data from other processes, the used MPI calls are collective
-    // operations. They need to be called on all processes, even if a process
-    // might not communicate any data.
-    auto exec = this->get_executor();
-    const auto comm = this->get_communicator();
-    auto num_cols = local_b->get_size()[1];
-    auto send_size = send_offsets_.back();
-    auto recv_size = recv_offsets_.back();
-    auto send_dim = dim<2>{static_cast<size_type>(send_size), num_cols};
-    auto recv_dim = dim<2>{static_cast<size_type>(recv_size), num_cols};
-    recv_buffer_.init(exec, recv_dim);
-    send_buffer_.init(exec, send_dim);
+    auto comm =
+        row_gatherer->get_collective_communicator()->get_base_communicator();
+    auto global_recv_dim =
+        dim<2>{static_cast<size_type>(row_gatherer->get_size()[0]), num_cols};
+    auto local_recv_dim = dim<2>{
+        static_cast<size_type>(
+            row_gatherer->get_collective_communicator()->get_recv_size()),
+        num_cols};
 
-    local_b->row_gather(&gather_idxs_, send_buffer_.get());
-
-    auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
-    if (use_host_buffer) {
-        host_recv_buffer_.init(exec->get_master(), recv_dim);
-        host_send_buffer_.init(exec->get_master(), send_dim);
-        host_send_buffer_->copy_from(send_buffer_.get());
-    }
-
-    mpi::contiguous_type type(num_cols, mpi::type_impl<ValueType>::get_type());
-    auto send_ptr = use_host_buffer ? host_send_buffer_->get_const_values()
-                                    : send_buffer_->get_const_values();
-    auto recv_ptr = use_host_buffer ? host_recv_buffer_->get_values()
-                                    : recv_buffer_->get_values();
-    exec->synchronize();
-#ifdef GINKGO_FORCE_SPMV_BLOCKING_COMM
-    comm.all_to_all_v(use_host_buffer ? exec->get_master() : exec, send_ptr,
-                      send_sizes_.data(), send_offsets_.data(), type.get(),
-                      recv_ptr, recv_sizes_.data(), recv_offsets_.data(),
-                      type.get());
-    return {};
-#else
-    return comm.i_all_to_all_v(
-        use_host_buffer ? exec->get_master() : exec, send_ptr,
-        send_sizes_.data(), send_offsets_.data(), type.get(), recv_ptr,
-        recv_sizes_.data(), recv_offsets_.data(), type.get());
-#endif
+    auto vector = buffer.template get<ValueType>(exec, comm, global_recv_dim,
+                                                 local_recv_dim);
+    auto host_vector = host_buffer.template get<ValueType>(
+        exec->get_master(), comm, global_recv_dim, local_recv_dim);
+    return std::make_pair(vector, host_vector);
 }
 
 
@@ -444,10 +447,14 @@ template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 void Matrix<ValueType, LocalIndexType, GlobalIndexType>::apply_impl(
     const LinOp* b, LinOp* x) const
 {
-    distributed::precision_dispatch_real_complex<ValueType>(
+    distributed::mixed_precision_dispatch_real_complex<ValueType>(
         [this](const auto dense_b, auto dense_x) {
+            using x_value_type =
+                typename std::decay_t<decltype(*dense_x)>::value_type;
+            using b_value_type =
+                typename std::decay_t<decltype(*dense_b)>::value_type;
             auto x_exec = dense_x->get_executor();
-            auto local_x = gko::matrix::Dense<ValueType>::create(
+            auto local_x = gko::matrix::Dense<x_value_type>::create(
                 x_exec, dense_x->get_local_vector()->get_size(),
                 gko::make_array_view(
                     x_exec,
@@ -455,18 +462,26 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::apply_impl(
                     dense_x->get_local_values()),
                 dense_x->get_local_vector()->get_stride());
 
+            auto exec = this->get_executor();
             auto comm = this->get_communicator();
-            auto req = this->communicate(dense_b->get_local_vector());
+            auto [recv_vector, host_recv_vector] =
+                init_recv_buffers<b_value_type>(
+                    exec, row_gatherer_.get(), dense_b->get_size()[1],
+                    recv_buffer_, host_recv_buffer_);
+            auto recv_ptr = mpi::requires_host_buffer(exec, comm)
+                                ? host_recv_vector.get()
+                                : recv_vector.get();
+            auto req = this->row_gatherer_->apply_async(dense_b, recv_ptr);
             local_mtx_->apply(dense_b->get_local_vector(), local_x);
             req.wait();
 
-            auto exec = this->get_executor();
-            auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
-            if (use_host_buffer) {
-                recv_buffer_->copy_from(host_recv_buffer_.get());
+            if (recv_ptr != recv_vector.get()) {
+                recv_vector->copy_from(host_recv_vector);
             }
-            non_local_mtx_->apply(one_scalar_.get(), recv_buffer_.get(),
-                                  one_scalar_.get(), local_x);
+            non_local_mtx_->apply(
+                one_scalar_.template get<ValueType>().get(),
+                recv_vector->get_local_vector(),
+                one_scalar_.template get<x_value_type>().get(), local_x);
         },
         b, x);
 }
@@ -476,11 +491,17 @@ template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 void Matrix<ValueType, LocalIndexType, GlobalIndexType>::apply_impl(
     const LinOp* alpha, const LinOp* b, const LinOp* beta, LinOp* x) const
 {
-    distributed::precision_dispatch_real_complex<ValueType>(
-        [this](const auto local_alpha, const auto dense_b,
-               const auto local_beta, auto dense_x) {
+    distributed::mixed_precision_dispatch_real_complex<ValueType>(
+        [this, alpha, beta](const auto dense_b, auto dense_x) {
+            using x_value_type =
+                typename std::decay_t<decltype(*dense_x)>::value_type;
+            using b_value_type =
+                typename std::decay_t<decltype(*dense_b)>::value_type;
             const auto x_exec = dense_x->get_executor();
-            auto local_x = gko::matrix::Dense<ValueType>::create(
+            auto local_alpha = gko::make_temporary_conversion<ValueType>(alpha);
+            auto local_beta =
+                gko::make_temporary_conversion<x_value_type>(beta);
+            auto local_x = gko::matrix::Dense<x_value_type>::create(
                 x_exec, dense_x->get_local_vector()->get_size(),
                 gko::make_array_view(
                     x_exec,
@@ -488,29 +509,116 @@ void Matrix<ValueType, LocalIndexType, GlobalIndexType>::apply_impl(
                     dense_x->get_local_values()),
                 dense_x->get_local_vector()->get_stride());
 
+            auto exec = this->get_executor();
             auto comm = this->get_communicator();
-            auto req = this->communicate(dense_b->get_local_vector());
-            local_mtx_->apply(local_alpha, dense_b->get_local_vector(),
-                              local_beta, local_x);
+            auto [recv_vector, host_recv_vector] =
+                init_recv_buffers<b_value_type>(
+                    exec, row_gatherer_.get(), dense_b->get_size()[1],
+                    recv_buffer_, host_recv_buffer_);
+            auto recv_ptr = mpi::requires_host_buffer(exec, comm)
+                                ? host_recv_vector.get()
+                                : recv_vector.get();
+            auto req = this->row_gatherer_->apply_async(dense_b, recv_ptr);
+            local_mtx_->apply(local_alpha.get(), dense_b->get_local_vector(),
+                              local_beta.get(), local_x);
             req.wait();
 
-            auto exec = this->get_executor();
-            auto use_host_buffer = mpi::requires_host_buffer(exec, comm);
-            if (use_host_buffer) {
-                recv_buffer_->copy_from(host_recv_buffer_.get());
+            if (recv_ptr != recv_vector.get()) {
+                recv_vector->copy_from(host_recv_vector);
             }
-            non_local_mtx_->apply(local_alpha, recv_buffer_.get(),
-                                  one_scalar_.get(), local_x);
+            non_local_mtx_->apply(
+                local_alpha.get(), recv_vector->get_local_vector(),
+                one_scalar_.template get<x_value_type>().get(), local_x);
         },
-        alpha, b, beta, x);
+        b, x);
+}
+
+
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::col_scale(
+    ptr_param<const global_vector_type> scaling_factors)
+{
+    GKO_ASSERT_CONFORMANT(this, scaling_factors.get());
+    GKO_ASSERT_EQ(scaling_factors->get_size()[1], 1);
+    auto exec = this->get_executor();
+    auto comm = this->get_communicator();
+    size_type n_local_cols = local_mtx_->get_size()[1];
+    size_type n_non_local_cols = non_local_mtx_->get_size()[1];
+
+    std::unique_ptr<global_vector_type> scaling_factors_single_stride;
+    auto scaling_stride = scaling_factors->get_stride();
+    if (scaling_stride != 1) {
+        scaling_factors_single_stride = global_vector_type::create(exec, comm);
+        scaling_factors_single_stride->copy_from(scaling_factors.get());
+    }
+    const global_vector_type* scaling_factors_ptr =
+        scaling_stride == 1 ? scaling_factors.get()
+                            : scaling_factors_single_stride.get();
+    const auto scale_diag = gko::matrix::Diagonal<ValueType>::create_const(
+        exec, n_local_cols,
+        make_const_array_view(exec, n_local_cols,
+                              scaling_factors_ptr->get_const_local_values()));
+
+    auto [recv_vector, host_recv_vector] = init_recv_buffers<ValueType>(
+        exec, row_gatherer_.get(), scaling_factors->get_size()[1], recv_buffer_,
+        host_recv_buffer_);
+    auto recv_ptr = mpi::requires_host_buffer(exec, comm)
+                        ? host_recv_vector.get()
+                        : recv_vector.get();
+
+    auto req = row_gatherer_->apply_async(scaling_factors_ptr, recv_ptr);
+    scale_diag->rapply(local_mtx_, local_mtx_);
+    req.wait();
+    if (n_non_local_cols > 0) {
+        if (recv_ptr != recv_vector.get()) {
+            recv_vector->copy_from(host_recv_vector);
+        }
+        const auto non_local_scale_diag =
+            gko::matrix::Diagonal<ValueType>::create_const(
+                exec, n_non_local_cols,
+                make_const_array_view(exec, n_non_local_cols,
+                                      recv_vector->get_const_local_values()));
+        non_local_scale_diag->rapply(non_local_mtx_, non_local_mtx_);
+    }
+}
+
+
+template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
+void Matrix<ValueType, LocalIndexType, GlobalIndexType>::row_scale(
+    ptr_param<const global_vector_type> scaling_factors)
+{
+    GKO_ASSERT_EQUAL_ROWS(this, scaling_factors.get());
+    GKO_ASSERT_EQ(scaling_factors->get_size()[1], 1);
+    auto exec = this->get_executor();
+    auto comm = this->get_communicator();
+    size_type n_local_rows = local_mtx_->get_size()[0];
+    std::unique_ptr<global_vector_type> scaling_factors_single_stride;
+    auto stride = scaling_factors->get_stride();
+    if (stride != 1) {
+        scaling_factors_single_stride = global_vector_type::create(exec, comm);
+        scaling_factors_single_stride->copy_from(scaling_factors.get());
+    }
+    const auto scale_values =
+        stride == 1 ? scaling_factors->get_const_local_values()
+                    : scaling_factors_single_stride->get_const_local_values();
+    const auto scale_diag = gko::matrix::Diagonal<ValueType>::create_const(
+        exec, n_local_rows,
+        make_const_array_view(exec, n_local_rows, scale_values));
+
+    scale_diag->apply(local_mtx_, local_mtx_);
+    scale_diag->apply(non_local_mtx_, non_local_mtx_);
 }
 
 
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(const Matrix& other)
-    : EnableDistributedLinOp<Matrix<value_type, local_index_type,
-                                    global_index_type>>{other.get_executor()},
-      DistributedBase{other.get_communicator()}
+    : EnableLinOp<Matrix<value_type, local_index_type,
+                         global_index_type>>{other.get_executor()},
+      DistributedBase{other.get_communicator()},
+      row_gatherer_{RowGatherer<LocalIndexType>::create(
+          other.get_executor(), other.get_communicator())},
+      imap_(other.get_executor()),
+      one_scalar_(other.get_executor(), 1.0)
 {
     *this = other;
 }
@@ -519,9 +627,13 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(const Matrix& other)
 template <typename ValueType, typename LocalIndexType, typename GlobalIndexType>
 Matrix<ValueType, LocalIndexType, GlobalIndexType>::Matrix(
     Matrix&& other) noexcept
-    : EnableDistributedLinOp<Matrix<value_type, local_index_type,
-                                    global_index_type>>{other.get_executor()},
-      DistributedBase{other.get_communicator()}
+    : EnableLinOp<Matrix<value_type, local_index_type,
+                         global_index_type>>{other.get_executor()},
+      DistributedBase{other.get_communicator()},
+      row_gatherer_{RowGatherer<LocalIndexType>::create(
+          other.get_executor(), other.get_communicator())},
+      imap_(other.get_executor()),
+      one_scalar_(other.get_executor(), 1.0)
 {
     *this = std::move(other);
 }
@@ -538,14 +650,8 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::operator=(
         this->set_size(other.get_size());
         local_mtx_->copy_from(other.local_mtx_);
         non_local_mtx_->copy_from(other.non_local_mtx_);
-        gather_idxs_ = other.gather_idxs_;
-        send_offsets_ = other.send_offsets_;
-        recv_offsets_ = other.recv_offsets_;
-        send_sizes_ = other.send_sizes_;
-        recv_sizes_ = other.recv_sizes_;
-        non_local_to_global_ = other.non_local_to_global_;
-        one_scalar_.init(this->get_executor(), dim<2>{1, 1});
-        one_scalar_->fill(one<value_type>());
+        row_gatherer_->copy_from(other.row_gatherer_);
+        imap_ = other.imap_;
     }
     return *this;
 }
@@ -562,14 +668,8 @@ Matrix<ValueType, LocalIndexType, GlobalIndexType>::operator=(Matrix&& other)
         other.set_size({});
         local_mtx_->move_from(other.local_mtx_);
         non_local_mtx_->move_from(other.non_local_mtx_);
-        gather_idxs_ = std::move(other.gather_idxs_);
-        send_offsets_ = std::move(other.send_offsets_);
-        recv_offsets_ = std::move(other.recv_offsets_);
-        send_sizes_ = std::move(other.send_sizes_);
-        recv_sizes_ = std::move(other.recv_sizes_);
-        non_local_to_global_ = std::move(other.non_local_to_global_);
-        one_scalar_.init(this->get_executor(), dim<2>{1, 1});
-        one_scalar_->fill(one<value_type>());
+        row_gatherer_->move_from(other.row_gatherer_);
+        imap_ = std::move(other.imap_);
     }
     return *this;
 }
