@@ -6,16 +6,12 @@
 #define GKO_CORE_FACTORIZATION_IC_KERNELS_HPP_
 
 
-#include <ginkgo/core/factorization/ic.hpp>
-
-
 #include <memory>
 
-
 #include <ginkgo/core/base/executor.hpp>
+#include <ginkgo/core/factorization/ic.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 
-
 #include "core/base/kernel_declaration.hpp"
 
 
@@ -23,13 +19,13 @@ namespace gko {
 namespace kernels {
 
 
-#define GKO_DECLARE_IC_COMPUTE_KERNEL(ValueType, IndexType)   \
-    void compute(std::shared_ptr<const DefaultExecutor> exec, \
-                 matrix::Csr<ValueType, IndexType>* system_matrix)
+#define GKO_DECLARE_IC_SPARSELIB_IC_KERNEL(ValueType, IndexType)   \
+    void sparselib_ic(std::shared_ptr<const DefaultExecutor> exec, \
+                      matrix::Csr<ValueType, IndexType>* system_matrix)
 
 #define GKO_DECLARE_ALL_AS_TEMPLATES                  \
     template <typename ValueType, typename IndexType> \
-    GKO_DECLARE_IC_COMPUTE_KERNEL(ValueType, IndexType)
+    GKO_DECLARE_IC_SPARSELIB_IC_KERNEL(ValueType, IndexType)
 
 
 GKO_DECLARE_FOR_ALL_EXECUTOR_NAMESPACES(ic_factorization,
