@@ -2,8 +2,7 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/log/performance_hint.hpp>
-
+#include "ginkgo/core/log/performance_hint.hpp"
 
 #include <iomanip>
 #include <sstream>
