@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -6,13 +6,11 @@
 #define GKO_CORE_MATRIX_CSR_KERNELS_HPP_
 
 
-#include <ginkgo/core/matrix/csr.hpp>
-
-
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/index_set.hpp>
 #include <ginkgo/core/base/types.hpp>
 #include <ginkgo/core/matrix/coo.hpp>
+#include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
 #include <ginkgo/core/matrix/ell.hpp>
@@ -20,7 +18,6 @@
 #include <ginkgo/core/matrix/sellp.hpp>
 #include <ginkgo/core/matrix/sparsity_csr.hpp>
 
-
 #include "core/base/kernel_declaration.hpp"
 #include "core/matrix/csr_lookup.hpp"
 
@@ -261,6 +258,12 @@ namespace kernels {
                           IndexType sample_size, IndexType* result)
 
 
+#define GKO_DECLARE_CSR_ROW_WISE_ABSOLUTE_SUM(ValueType, IndexType)           \
+    void row_wise_absolute_sum(std::shared_ptr<const DefaultExecutor> exec,   \
+                               const matrix::Csr<ValueType, IndexType>* orig, \
+                               array<ValueType>& sum)
+
+
 #define GKO_DECLARE_ALL_AS_TEMPLATES                                        \
     template <typename MatrixValueType, typename InputValueType,            \
               typename OutputValueType, typename IndexType>                 \
@@ -339,7 +342,9 @@ namespace kernels {
     template <typename IndexType>                                           \
     GKO_DECLARE_CSR_BUILD_LOOKUP_KERNEL(IndexType);                         \
     template <typename IndexType>                                           \
-    GKO_DECLARE_CSR_BENCHMARK_LOOKUP_KERNEL(IndexType)
+    GKO_DECLARE_CSR_BENCHMARK_LOOKUP_KERNEL(IndexType);                     \
+    template <typename ValueType, typename IndexType>                       \
+    GKO_DECLARE_CSR_ROW_WISE_ABSOLUTE_SUM(ValueType, IndexType)
 
 
 GKO_DECLARE_FOR_ALL_EXECUTOR_NAMESPACES(csr, GKO_DECLARE_ALL_AS_TEMPLATES);
