@@ -1,9 +1,8 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/matrix/permutation.hpp>
-
+#include "ginkgo/core/matrix/permutation.hpp"
 
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/executor.hpp>
@@ -11,7 +10,6 @@
 #include <ginkgo/core/base/temporary_clone.hpp>
 #include <ginkgo/core/base/utils_helper.hpp>
 
-
 #include "core/base/dispatch_helper.hpp"
 #include "core/matrix/permutation_kernels.hpp"
 
@@ -269,8 +267,14 @@ void dispatch_dense(const LinOp* op, Functor fn)
 {
     using matrix::Dense;
     using std::complex;
-    run<Dense, double, float, std::complex<double>, std::complex<float>>(op,
-                                                                         fn);
+    run<Dense,
+#if GINKGO_ENABLE_HALF
+        gko::float16, std::complex<gko::float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        gko::bfloat16, std::complex<gko::bfloat16>,
+#endif
+        double, float, std::complex<double>, std::complex<float>>(op, fn);
 }
 
 
