@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -8,14 +8,12 @@
 
 #include <memory>
 
-
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/coo.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
 
-
 #include "core/base/kernel_declaration.hpp"
 
 
@@ -71,11 +69,6 @@ namespace pgm {
         const matrix::Diagonal<ValueType>* diag, array<IndexType>& agg, \
         array<IndexType>& intermediate_agg)
 
-#define GKO_DECLARE_PGM_SORT_ROW_MAJOR(ValueType, IndexType)         \
-    void sort_row_major(std::shared_ptr<const DefaultExecutor> exec, \
-                        size_type nnz, IndexType* row_idxs,          \
-                        IndexType* col_idxs, ValueType* vals)
-
 #define GKO_DECLARE_PGM_COMPUTE_COARSE_COO(ValueType, IndexType)              \
     void compute_coarse_coo(std::shared_ptr<const DefaultExecutor> exec,      \
                             size_type fine_nnz, const IndexType* row_idxs,    \
@@ -108,8 +101,6 @@ namespace pgm {
     template <typename ValueType, typename IndexType>              \
     GKO_DECLARE_PGM_ASSIGN_TO_EXIST_AGG(ValueType, IndexType);     \
     template <typename ValueType, typename IndexType>              \
-    GKO_DECLARE_PGM_SORT_ROW_MAJOR(ValueType, IndexType);          \
-    template <typename ValueType, typename IndexType>              \
     GKO_DECLARE_PGM_COMPUTE_COARSE_COO(ValueType, IndexType);      \
     template <typename IndexType>                                  \
     GKO_DECLARE_PGM_GATHER_INDEX(IndexType)
