@@ -1,18 +1,17 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/solver/multigrid.hpp>
-
+#include "ginkgo/core/solver/multigrid.hpp"
 
 #include <complex>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/base/lin_op.hpp>
 #include <ginkgo/core/base/math.hpp>
+#include <ginkgo/core/base/types.hpp>
 #include <ginkgo/core/base/utils.hpp>
 #include <ginkgo/core/base/utils_helper.hpp>
 #include <ginkgo/core/distributed/matrix.hpp>
@@ -27,7 +26,6 @@
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 
-
 #include "core/base/dispatch_helper.hpp"
 #include "core/components/fill_array_kernels.hpp"
 #include "core/config/config_helper.hpp"
@@ -203,7 +201,8 @@ namespace detail {
  *
  * @note it should only be used internally
  */
-struct MultigridState {
+class MultigridState {
+public:
     MultigridState() : nrhs{static_cast<size_type>(-1)} {}
 
     /**
@@ -320,6 +319,12 @@ void MultigridState::generate(const LinOp* system_matrix_in,
         auto mg_level = mg_level_list.at(i);
 
         run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+            float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+            bfloat16, std::complex<bfloat16>,
+#endif
             std::complex<float>, std::complex<double>>(
             mg_level,
             [&, this](auto mg_level, auto i, auto cycle, auto current_nrows,
@@ -352,6 +357,7 @@ void MultigridState::generate(const LinOp* system_matrix_in,
                     this->allocate_memory<VectorType>(
                         i, cycle, current_comm, next_comm, current_nrows,
                         next_nrows, current_local_nrows, next_local_nrows);
+
                 } else
 #endif
                 {
@@ -458,6 +464,12 @@ void MultigridState::run_mg_cycle(multigrid::cycle cycle, size_type level,
     }
     auto mg_level = multigrid->get_mg_level_list().at(level);
     run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+        float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        bfloat16, std::complex<bfloat16>,
+#endif
         std::complex<float>, std::complex<double>>(
         mg_level, [&, this](auto mg_level) {
 #if GINKGO_BUILD_MPI
@@ -489,7 +501,7 @@ void MultigridState::run_cycle(multigrid::cycle cycle, size_type level,
 
     auto r = r_list.at(level);
     auto g = g_list.at(level);
-    auto e = as<VectorType>(e_list.at(level));
+    auto e = e_list.at(level);
     // get mg_level
     auto mg_level = multigrid->get_mg_level_list().at(level);
     // get the pre_smoother
@@ -540,7 +552,7 @@ void MultigridState::run_cycle(multigrid::cycle cycle, size_type level,
     // next level
     if (level + 1 == total_level) {
         // the coarsest solver use the last level valuetype
-        e->fill(zero<value_type>());
+        as<VectorType>(e)->fill(zero<value_type>());
     }
     auto next_level_matrix =
         (level + 1 < total_level)
@@ -589,6 +601,7 @@ void MultigridState::run_cycle(multigrid::cycle cycle, size_type level,
     }
 }
 
+
 }  // namespace detail
 }  // namespace multigrid
 
@@ -707,6 +720,12 @@ void Multigrid::generate()
         }
 
         run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+            float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+            bfloat16, std::complex<bfloat16>,
+#endif
             std::complex<float>, std::complex<double>>(
             mg_level,
             [this](auto mg_level, auto index, auto matrix) {
@@ -745,6 +764,12 @@ void Multigrid::generate()
 
     // generate coarsest solver
     run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+        float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        bfloat16, std::complex<bfloat16>,
+#endif
         std::complex<float>, std::complex<double>>(
         last_mg_level,
         [this](auto mg_level, auto level, auto matrix) {
@@ -754,7 +779,6 @@ void Multigrid::generate()
             // default coarse grid solver, direct LU
             // TODO: maybe remove fixed index type
             auto gen_default_solver = [&]() -> std::unique_ptr<LinOp> {
-        // TODO: unify when dpcpp supports direct solver
 #if GINKGO_BUILD_MPI
                 if (gko::detail::is_distributed(matrix.get())) {
                     using absolute_value_type = remove_complex<value_type>;
@@ -790,6 +814,7 @@ void Multigrid::generate()
                     });
                 }
 #endif
+                // TODO: unify when dpcpp supports direct solver
                 if (dynamic_cast<const DpcppExecutor*>(exec.get())) {
                     using absolute_value_type = remove_complex<value_type>;
                     return solver::Gmres<value_type>::build()
@@ -862,6 +887,12 @@ void Multigrid::apply_with_initial_guess_impl(const LinOp* b, LinOp* x,
     };
     auto first_mg_level = this->get_mg_level_list().front();
     run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+        float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        bfloat16, std::complex<bfloat16>,
+#endif
         std::complex<float>, std::complex<double>>(first_mg_level, lambda, b,
                                                    x);
 }
@@ -901,6 +932,12 @@ void Multigrid::apply_with_initial_guess_impl(const LinOp* alpha,
     };
     auto first_mg_level = this->get_mg_level_list().front();
     run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+        float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        bfloat16, std::complex<bfloat16>,
+#endif
         std::complex<float>, std::complex<double>>(first_mg_level, lambda,
                                                    alpha, b, beta, x);
 }
@@ -966,6 +1003,12 @@ void Multigrid::apply_dense_impl(const VectorType* b, VectorType* x,
     auto first_mg_level = this->get_mg_level_list().front();
 
     run<gko::multigrid::EnableMultigridLevel, float, double,
+#if GINKGO_ENABLE_HALF
+        float16, std::complex<float16>,
+#endif
+#if GINKGO_ENABLE_BFLOAT16
+        bfloat16, std::complex<bfloat16>,
+#endif
         std::complex<float>, std::complex<double>>(first_mg_level, lambda, b,
                                                    x);
 }
