@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,6 +11,7 @@
 #include <ginkgo/core/config/config.hpp>
 #include <ginkgo/core/distributed/preconditioner/schwarz.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
+#include <ginkgo/core/multigrid/pgm.hpp>
 #include <ginkgo/core/preconditioner/gauss_seidel.hpp>
 #include <ginkgo/core/preconditioner/ic.hpp>
 #include <ginkgo/core/preconditioner/ilu.hpp>
@@ -30,6 +31,8 @@ using namespace gko::config;
 
 using DummyIr = gko::solver::Ir<float>;
 
+using DummyMgLevel = gko::multigrid::Pgm<float, int>;
+
 
 template <typename ChangedType, typename DefaultType>
 struct PreconditionerConfigTest {
@@ -43,9 +46,8 @@ struct PreconditionerConfigTest {
 };
 
 
-struct Ic : PreconditionerConfigTest<
-                ::gko::preconditioner::Ic<DummyIr, int>,
-                ::gko::preconditioner::Ic<gko::solver::LowerTrs<>, int>> {
+struct Ic : PreconditionerConfigTest<::gko::preconditioner::Ic<float, int>,
+                                     ::gko::preconditioner::Ic<double, int>> {
     static pnode::map_type setup_base()
     {
         return {{"type", pnode{"preconditioner::Ic"}}};
@@ -54,7 +56,6 @@ struct Ic : PreconditionerConfigTest<
     static void change_template(pnode::map_type& config_map)
     {
         config_map["value_type"] = pnode{"float32"};
-        config_map["l_solver_type"] = pnode{"solver::Ir"};
     }
 
     template <bool from_reg, typename ParamType>
@@ -63,9 +64,9 @@ struct Ic : PreconditionerConfigTest<
     {
         if (from_reg) {
             config_map["l_solver"] = pnode{"l_solver"};
-            param.with_l_solver(detail::registry_accessor::get_data<
-                                typename changed_type::l_solver_type::Factory>(
-                reg, "l_solver"));
+            param.with_l_solver(
+                detail::registry_accessor::get_data<gko::LinOpFactory>(
+                    reg, "l_solver"));
             config_map["factorization"] = pnode{"factorization"};
             param.with_factorization(
                 detail::registry_accessor::get_data<gko::LinOpFactory>(
@@ -73,7 +74,7 @@ struct Ic : PreconditionerConfigTest<
         } else {
             config_map["l_solver"] = pnode{{{"type", pnode{"solver::Ir"}},
                                             {"value_type", pnode{"float32"}}}};
-            param.with_l_solver(changed_type::l_solver_type::build().on(exec));
+            param.with_l_solver(DummyIr::build().on(exec));
             config_map["factorization"] =
                 pnode{{{"type", pnode{"solver::Ir"}},
                        {"value_type", pnode{"float32"}}}};
@@ -105,11 +106,9 @@ struct Ic : PreconditionerConfigTest<
 };
 
 
-struct Ilu
-    : PreconditionerConfigTest<
-          ::gko::preconditioner::Ilu<DummyIr, DummyIr, true, int>,
-          ::gko::preconditioner::Ilu<gko::solver::LowerTrs<>,
-                                     gko::solver::UpperTrs<>, false, int>> {
+struct Ilu : PreconditionerConfigTest<
+                 ::gko::preconditioner::Ilu<float, float, true, int>,
+                 ::gko::preconditioner::Ilu<double, double, false, int>> {
     static pnode::map_type setup_base()
     {
         return {{"type", pnode{"preconditioner::Ilu"}}};
@@ -118,7 +117,6 @@ struct Ilu
     static void change_template(pnode::map_type& config_map)
     {
         config_map["value_type"] = pnode{"float32"};
-        config_map["l_solver_type"] = pnode{"solver::Ir"};
         config_map["reverse_apply"] = pnode{true};
     }
 
@@ -128,13 +126,13 @@ struct Ilu
     {
         if (from_reg) {
             config_map["l_solver"] = pnode{"l_solver"};
-            param.with_l_solver(detail::registry_accessor::get_data<
-                                typename changed_type::l_solver_type::Factory>(
-                reg, "l_solver"));
+            param.with_l_solver(
+                detail::registry_accessor::get_data<gko::LinOpFactory>(
+                    reg, "l_solver"));
             config_map["u_solver"] = pnode{"u_solver"};
-            param.with_u_solver(detail::registry_accessor::get_data<
-                                typename changed_type::u_solver_type::Factory>(
-                reg, "u_solver"));
+            param.with_u_solver(
+                detail::registry_accessor::get_data<gko::LinOpFactory>(
+                    reg, "u_solver"));
             config_map["factorization"] = pnode{"factorization"};
             param.with_factorization(
                 detail::registry_accessor::get_data<gko::LinOpFactory>(
@@ -142,10 +140,10 @@ struct Ilu
         } else {
             config_map["l_solver"] = pnode{{{"type", pnode{"solver::Ir"}},
                                             {"value_type", pnode{"float32"}}}};
-            param.with_l_solver(changed_type::l_solver_type::build().on(exec));
+            param.with_l_solver(DummyIr::build().on(exec));
             config_map["u_solver"] = pnode{{{"type", pnode{"solver::Ir"}},
                                             {"value_type", pnode{"float32"}}}};
-            param.with_u_solver(changed_type::u_solver_type::build().on(exec));
+            param.with_u_solver(DummyIr::build().on(exec));
             config_map["factorization"] =
                 pnode{{{"type", pnode{"solver::Ir"}},
                        {"value_type", pnode{"float32"}}}};
@@ -279,6 +277,8 @@ struct Jacobi
         config_map["accuracy"] = pnode{1e-2};
         param.with_accuracy(
             gko::remove_complex<typename changed_type::value_type>{1e-2});
+        config_map["aggregate_l1"] = pnode{true};
+        param.with_aggregate_l1(true);
     }
 
     template <bool from_reg, typename AnswerType>
@@ -293,6 +293,7 @@ struct Jacobi
         ASSERT_EQ(res_param.max_block_stride, ans_param.max_block_stride);
         ASSERT_EQ(res_param.skip_sorting, ans_param.skip_sorting);
         GKO_ASSERT_ARRAY_EQ(res_param.block_pointers, ans_param.block_pointers);
+        ASSERT_EQ(res_param.aggregate_l1, ans_param.aggregate_l1);
 
         ASSERT_EQ(res_so.is_block_wise, ans_so.is_block_wise);
         ASSERT_EQ(res_so.of_all_blocks, ans_so.of_all_blocks);
@@ -399,7 +400,6 @@ struct GaussSeidel
 
 #if GINKGO_BUILD_MPI
 
-
 struct Schwarz
     : PreconditionerConfigTest<
           ::gko::experimental::distributed::preconditioner::Schwarz<float, int,
@@ -425,15 +425,34 @@ struct Schwarz
             param.with_local_solver(
                 detail::registry_accessor::get_data<gko::LinOpFactory>(
                     reg, "solver"));
+            config_map["coarse_solver"] = pnode{"solver"};
+            param.with_coarse_solver(
+                detail::registry_accessor::get_data<gko::LinOpFactory>(
+                    reg, "solver"));
+            config_map["coarse_level"] = pnode{"c_level"};
+            param.with_coarse_level(
+                detail::registry_accessor::get_data<gko::LinOpFactory>(
+                    reg, "c_level"));
         } else {
             config_map["local_solver"] =
                 pnode{{{"type", pnode{"solver::Ir"}},
                        {"value_type", pnode{"float32"}}}};
             param.with_local_solver(DummyIr::build().on(exec));
+            config_map["coarse_solver"] =
+                pnode{{{"type", pnode{"solver::Ir"}},
+                       {"value_type", pnode{"float32"}}}};
+            param.with_coarse_solver(DummyIr::build().on(exec));
+            config_map["coarse_level"] =
+                pnode{{{"type", pnode{"multigrid::Pgm"}}}};
+            param.with_coarse_level(DummyMgLevel::build().on(exec));
         }
         config_map["generated_local_solver"] = pnode{"linop"};
         param.with_generated_local_solver(
             detail::registry_accessor::get_data<gko::LinOp>(reg, "linop"));
+        config_map["l1_smoother"] = pnode{true};
+        param.with_l1_smoother(true);
+        config_map["coarse_weight"] = pnode{0.1};
+        param.with_coarse_weight(decltype(param.coarse_weight){0.1});
     }
 
     template <bool from_reg, typename AnswerType>
@@ -444,14 +463,25 @@ struct Schwarz
 
         if (from_reg) {
             ASSERT_EQ(res_param.local_solver, ans_param.local_solver);
+            ASSERT_EQ(res_param.coarse_solver, ans_param.coarse_solver);
+            ASSERT_EQ(res_param.coarse_level, ans_param.coarse_level);
         } else {
             ASSERT_NE(
                 std::dynamic_pointer_cast<const typename DummyIr::Factory>(
                     res_param.local_solver),
                 nullptr);
+            ASSERT_NE(
+                std::dynamic_pointer_cast<const typename DummyIr::Factory>(
+                    res_param.coarse_solver),
+                nullptr);
+            ASSERT_NE(
+                std::dynamic_pointer_cast<const typename DummyMgLevel::Factory>(
+                    res_param.coarse_level),
+                nullptr);
         }
         ASSERT_EQ(res_param.generated_local_solver,
                   ans_param.generated_local_solver);
+        ASSERT_EQ(res_param.coarse_weight, ans_param.coarse_weight);
     }
 };
 
@@ -472,9 +502,11 @@ protected:
           u_solver(DummyIr::build().on(exec)),
           factorization(DummyIr::build().on(exec)),
           linop(gko::matrix::Dense<>::create(exec)),
+          coarse_level(DummyMgLevel::build().on(exec)),
           reg()
     {
         reg.emplace("solver", solver_factory);
+        reg.emplace("c_level", coarse_level);
         reg.emplace("l_solver", l_solver);
         reg.emplace("u_solver", u_solver);
         reg.emplace("factorization", factorization);
@@ -487,6 +519,7 @@ protected:
     std::shared_ptr<typename DummyIr::Factory> l_solver;
     std::shared_ptr<typename DummyIr::Factory> u_solver;
     std::shared_ptr<typename DummyIr::Factory> factorization;
+    std::shared_ptr<typename DummyMgLevel::Factory> coarse_level;
     std::shared_ptr<gko::LinOp> linop;
     registry reg;
 };
