@@ -2,17 +2,13 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/matrix/row_gatherer.hpp>
-
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/base/range.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
-
+#include <ginkgo/core/matrix/row_gatherer.hpp>
 
 #include "core/test/utils.hpp"
 #include "core/test/utils/assertions.hpp"
@@ -69,7 +65,7 @@ protected:
     std::unique_ptr<OutVec> out;
 };
 
-TYPED_TEST_SUITE(RowGatherer, gko::test::TwoValueIndexType,
+TYPED_TEST_SUITE(RowGatherer, gko::test::TwoValueIndexTypes,
                  TupleTypenameNameGenerator);
 
 
