@@ -6,12 +6,10 @@
 #define GKO_CUDA_BASE_CONFIG_HPP_
 
 
+#include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/types.hpp>
 
 
-#include "cuda/base/math.hpp"
-
-
 namespace gko {
 namespace kernels {
 namespace cuda {
