@@ -2,15 +2,12 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/base/stream.hpp>
-
+#include "ginkgo/core/base/stream.hpp"
 
 #include <cuda_runtime.h>
 
-
 #include <ginkgo/core/base/exception_helpers.hpp>
 
-
 #include "cuda/base/scoped_device_id.hpp"
 
 
