@@ -4,26 +4,25 @@
 
 #include "core/base/batch_multi_vector_kernels.hpp"
 
-
 #include <algorithm>
 
-
-#include <CL/sycl.hpp>
-
+#include <sycl/sycl.hpp>
 
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/batch_multi_vector.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/math.hpp>
 
-
 #include "core/base/batch_struct.hpp"
 #include "core/components/prefix_sum_kernels.hpp"
+#include "dpcpp/base/batch_multi_vector_kernels.hpp"
 #include "dpcpp/base/batch_struct.hpp"
 #include "dpcpp/base/config.hpp"
 #include "dpcpp/base/dim3.dp.hpp"
 #include "dpcpp/base/dpct.hpp"
 #include "dpcpp/base/helper.hpp"
+#include "dpcpp/base/math.hpp"
+#include "dpcpp/base/types.hpp"
 #include "dpcpp/components/cooperative_groups.dp.hpp"
 #include "dpcpp/components/intrinsics.dp.hpp"
 #include "dpcpp/components/reduction.dp.hpp"
@@ -33,21 +32,13 @@
 namespace gko {
 namespace kernels {
 namespace dpcpp {
-/**
- * @brief The MultiVector matrix format namespace.
- * @ref MultiVector
- * @ingroup batch_multi_vector
- */
 namespace batch_multi_vector {
 
 
-#include "dpcpp/base/batch_multi_vector_kernels.hpp.inc"
-
-
 template <typename ValueType>
 void scale(std::shared_ptr<const DefaultExecutor> exec,
-           const batch::MultiVector<ValueType>* const alpha,
-           batch::MultiVector<ValueType>* const x)
+           const batch::MultiVector<ValueType>* alpha,
+           batch::MultiVector<ValueType>* x)
 {
     const auto alpha_ub = get_batch_struct(alpha);
     const auto x_ub = get_batch_struct(x);
@@ -75,7 +66,7 @@ void scale(std::shared_ptr<const DefaultExecutor> exec,
                     const auto alpha_b =
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
-                    scale_kernel(
+                    batch_single_kernels::scale_kernel(
                         alpha_b, x_b, item_ct1,
                         [](int row, int col, int stride) { return 0; });
                 });
@@ -89,10 +80,11 @@ void scale(std::shared_ptr<const DefaultExecutor> exec,
                     const auto alpha_b =
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
-                    scale_kernel(alpha_b, x_b, item_ct1,
-                                 [](int row, int col, int stride) {
-                                     return row * stride + col;
-                                 });
+                    batch_single_kernels::scale_kernel(
+                        alpha_b, x_b, item_ct1,
+                        [](int row, int col, int stride) {
+                            return row * stride + col;
+                        });
                 });
         });
     } else {
@@ -104,7 +96,7 @@ void scale(std::shared_ptr<const DefaultExecutor> exec,
                     const auto alpha_b =
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
-                    scale_kernel(
+                    batch_single_kernels::scale_kernel(
                         alpha_b, x_b, item_ct1,
                         [](int row, int col, int stride) { return col; });
                 });
@@ -118,9 +110,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 
 template <typename ValueType>
 void add_scaled(std::shared_ptr<const DefaultExecutor> exec,
-                const batch::MultiVector<ValueType>* const alpha,
-                const batch::MultiVector<ValueType>* const x,
-                batch::MultiVector<ValueType>* const y)
+                const batch::MultiVector<ValueType>* alpha,
+                const batch::MultiVector<ValueType>* x,
+                batch::MultiVector<ValueType>* y)
 {
     constexpr int max_subgroup_size = config::warp_size;
     const int num_rows = x->get_common_size()[0];
@@ -148,8 +140,9 @@ void add_scaled(std::shared_ptr<const DefaultExecutor> exec,
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
                     const auto y_b = batch::extract_batch_item(y_ub, group_id);
-                    add_scaled_kernel(alpha_b, x_b, y_b, item_ct1,
-                                      [](auto col) { return 0; });
+                    batch_single_kernels::add_scaled_kernel(
+                        alpha_b, x_b, y_b, item_ct1,
+                        [](auto col) { return 0; });
                 });
         });
     } else {
@@ -162,8 +155,9 @@ void add_scaled(std::shared_ptr<const DefaultExecutor> exec,
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
                     const auto y_b = batch::extract_batch_item(y_ub, group_id);
-                    add_scaled_kernel(alpha_b, x_b, y_b, item_ct1,
-                                      [](auto col) { return col; });
+                    batch_single_kernels::add_scaled_kernel(
+                        alpha_b, x_b, y_b, item_ct1,
+                        [](auto col) { return col; });
                 });
         });
     }
@@ -175,9 +169,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 
 template <typename ValueType>
 void compute_dot(std::shared_ptr<const DefaultExecutor> exec,
-                 const batch::MultiVector<ValueType>* const x,
-                 const batch::MultiVector<ValueType>* const y,
-                 batch::MultiVector<ValueType>* const result)
+                 const batch::MultiVector<ValueType>* x,
+                 const batch::MultiVector<ValueType>* y,
+                 batch::MultiVector<ValueType>* result)
 {
     const auto x_ub = get_batch_struct(x);
     const auto y_ub = get_batch_struct(y);
@@ -210,7 +204,7 @@ void compute_dot(std::shared_ptr<const DefaultExecutor> exec,
                             batch::extract_batch_item(y_ub, group_id);
                         const auto res_b =
                             batch::extract_batch_item(res_ub, group_id);
-                        single_rhs_compute_conj_dot_sg(
+                        batch_single_kernels::single_rhs_compute_conj_dot_sg(
                             x_b.num_rows, x_b.values, y_b.values,
                             res_b.values[0], item_ct1);
                     });
@@ -230,7 +224,7 @@ void compute_dot(std::shared_ptr<const DefaultExecutor> exec,
                             batch::extract_batch_item(y_ub, group_id);
                         const auto res_b =
                             batch::extract_batch_item(res_ub, group_id);
-                        compute_gen_dot_product_kernel(
+                        batch_single_kernels::compute_gen_dot_product_kernel(
                             x_b, y_b, res_b, item_ct1,
                             [](auto val) { return val; });
                     });
@@ -244,9 +238,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 
 template <typename ValueType>
 void compute_conj_dot(std::shared_ptr<const DefaultExecutor> exec,
-                      const batch::MultiVector<ValueType>* const x,
-                      const batch::MultiVector<ValueType>* const y,
-                      batch::MultiVector<ValueType>* const result)
+                      const batch::MultiVector<ValueType>* x,
+                      const batch::MultiVector<ValueType>* y,
+                      batch::MultiVector<ValueType>* result)
 {
     const auto x_ub = get_batch_struct(x);
     const auto y_ub = get_batch_struct(y);
@@ -276,7 +270,7 @@ void compute_conj_dot(std::shared_ptr<const DefaultExecutor> exec,
                     const auto y_b = batch::extract_batch_item(y_ub, group_id);
                     const auto res_b =
                         batch::extract_batch_item(res_ub, group_id);
-                    compute_gen_dot_product_kernel(
+                    batch_single_kernels::compute_gen_dot_product_kernel(
                         x_b, y_b, res_b, item_ct1,
                         [](auto val) { return conj(val); });
                 });
@@ -289,8 +283,8 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 
 template <typename ValueType>
 void compute_norm2(std::shared_ptr<const DefaultExecutor> exec,
-                   const batch::MultiVector<ValueType>* const x,
-                   batch::MultiVector<remove_complex<ValueType>>* const result)
+                   const batch::MultiVector<ValueType>* x,
+                   batch::MultiVector<remove_complex<ValueType>>* result)
 {
     const auto x_ub = get_batch_struct(x);
     const auto res_ub = get_batch_struct(result);
@@ -312,17 +306,16 @@ void compute_norm2(std::shared_ptr<const DefaultExecutor> exec,
         exec->get_queue()->submit([&](sycl::handler& cgh) {
             cgh.parallel_for(
                 sycl_nd_range(grid, block),
-                [=](sycl::nd_item<3> item_ct1)
-                    [[sycl::reqd_sub_group_size(max_subgroup_size)]] {
-                        auto group = item_ct1.get_group();
-                        auto group_id = group.get_group_linear_id();
-                        const auto x_b =
-                            batch::extract_batch_item(x_ub, group_id);
-                        const auto res_b =
-                            batch::extract_batch_item(res_ub, group_id);
-                        single_rhs_compute_norm2_sg(x_b.num_rows, x_b.values,
-                                                    res_b.values[0], item_ct1);
-                    });
+                [=](sycl::nd_item<3> item_ct1) [[sycl::reqd_sub_group_size(
+                    max_subgroup_size)]] {
+                    auto group = item_ct1.get_group();
+                    auto group_id = group.get_group_linear_id();
+                    const auto x_b = batch::extract_batch_item(x_ub, group_id);
+                    const auto res_b =
+                        batch::extract_batch_item(res_ub, group_id);
+                    batch_single_kernels::single_rhs_compute_norm2_sg(
+                        x_b.num_rows, x_b.values, res_b.values[0], item_ct1);
+                });
         });
     } else {
         exec->get_queue()->submit([&](sycl::handler& cgh) {
@@ -336,7 +329,8 @@ void compute_norm2(std::shared_ptr<const DefaultExecutor> exec,
                             batch::extract_batch_item(x_ub, group_id);
                         const auto res_b =
                             batch::extract_batch_item(res_ub, group_id);
-                        compute_norm2_kernel(x_b, res_b, item_ct1);
+                        batch_single_kernels::compute_norm2_kernel(x_b, res_b,
+                                                                   item_ct1);
                     });
         });
     }
@@ -375,7 +369,7 @@ void copy(std::shared_ptr<const DefaultExecutor> exec,
                 const auto x_b = batch::extract_batch_item(x_ub, group_id);
                 const auto result_b =
                     batch::extract_batch_item(result_ub, group_id);
-                copy_kernel(x_b, result_b, item_ct1);
+                batch_single_kernels::copy_kernel(x_b, result_b, item_ct1);
             });
     });
 }
