@@ -2,11 +2,10 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <CL/sycl.hpp>
-
-
 #include "dpcpp/base/helper.hpp"
 
+#include <sycl/sycl.hpp>
+
 
 namespace gko {
 namespace kernels {
