@@ -4,44 +4,35 @@
 
 #include "core/matrix/batch_ell_kernels.hpp"
 
-
 #include <algorithm>
 
-
-#include <CL/sycl.hpp>
-
+#include <sycl/sycl.hpp>
 
 #include <ginkgo/core/base/batch_multi_vector.hpp>
 #include <ginkgo/core/matrix/batch_ell.hpp>
 
-
 #include "core/base/batch_struct.hpp"
 #include "core/matrix/batch_struct.hpp"
 #include "dpcpp/base/batch_struct.hpp"
 #include "dpcpp/base/dim3.dp.hpp"
 #include "dpcpp/base/dpct.hpp"
 #include "dpcpp/base/helper.hpp"
+#include "dpcpp/base/math.hpp"
+#include "dpcpp/base/types.hpp"
 #include "dpcpp/components/cooperative_groups.dp.hpp"
 #include "dpcpp/components/intrinsics.dp.hpp"
 #include "dpcpp/components/reduction.dp.hpp"
 #include "dpcpp/components/thread_ids.dp.hpp"
+#include "dpcpp/matrix/batch_ell_kernels.hpp"
 #include "dpcpp/matrix/batch_struct.hpp"
 
 
 namespace gko {
 namespace kernels {
 namespace dpcpp {
-/**
- * @brief The Ell matrix format namespace.
- * @ref Ell
- * @ingroup batch_ell
- */
 namespace batch_ell {
 
 
-#include "dpcpp/matrix/batch_ell_kernels.hpp.inc"
-
-
 template <typename ValueType, typename IndexType>
 void simple_apply(std::shared_ptr<const DefaultExecutor> exec,
                   const batch::matrix::Ell<ValueType, IndexType>* mat,
@@ -78,8 +69,8 @@ void simple_apply(std::shared_ptr<const DefaultExecutor> exec,
                         batch::matrix::extract_batch_item(mat_ub, group_id);
                     const auto b_b = batch::extract_batch_item(b_ub, group_id);
                     const auto x_b = batch::extract_batch_item(x_ub, group_id);
-                    simple_apply_kernel(mat_b, b_b.values, x_b.values,
-                                        item_ct1);
+                    batch_single_kernels::simple_apply(mat_b, b_b.values,
+                                                       x_b.values, item_ct1);
                 });
     });
 }
@@ -131,9 +122,9 @@ void advanced_apply(std::shared_ptr<const DefaultExecutor> exec,
                         batch::extract_batch_item(alpha_ub, group_id);
                     const auto beta_b =
                         batch::extract_batch_item(beta_ub, group_id);
-                    advanced_apply_kernel(alpha_b.values[0], mat_b, b_b.values,
-                                          beta_b.values[0], x_b.values,
-                                          item_ct1);
+                    batch_single_kernels::advanced_apply(
+                        alpha_b.values[0], mat_b, b_b.values, beta_b.values[0],
+                        x_b.values, item_ct1);
                 });
     });
 }
@@ -147,8 +138,8 @@ void scale(std::shared_ptr<const DefaultExecutor> exec,
            const array<ValueType>* col_scale, const array<ValueType>* row_scale,
            batch::matrix::Ell<ValueType, IndexType>* input)
 {
-    const auto col_scale_vals = col_scale->get_const_data();
-    const auto row_scale_vals = row_scale->get_const_data();
+    const auto col_scale_vals = as_device_type(col_scale->get_const_data());
+    const auto row_scale_vals = as_device_type(row_scale->get_const_data());
     const auto num_rows = static_cast<int>(input->get_common_size()[0]);
     const auto num_cols = static_cast<int>(input->get_common_size()[1]);
     auto mat_ub = get_batch_struct(input);
@@ -175,7 +166,8 @@ void scale(std::shared_ptr<const DefaultExecutor> exec,
                         row_scale_vals + num_rows * group_id;
                     auto mat_item =
                         batch::matrix::extract_batch_item(mat_ub, group_id);
-                    scale_kernel(col_scale_b, row_scale_b, mat_item, item_ct1);
+                    batch_single_kernels::scale(col_scale_b, row_scale_b,
+                                                mat_item, item_ct1);
                 });
     });
 }
@@ -216,7 +208,7 @@ void add_scaled_identity(std::shared_ptr<const DefaultExecutor> exec,
                         gko::batch::extract_batch_item(beta_ub, group_id);
                     const auto mat_b = gko::batch::matrix::extract_batch_item(
                         mat_ub, group_id);
-                    add_scaled_identity_kernel(
+                    batch_single_kernels::add_scaled_identity(
                         alpha_b.values[0], beta_b.values[0], mat_b, item_ct1);
                 });
     });
