@@ -1,13 +1,12 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/matrix/dense_kernels.hpp"
 
-
-#include <CL/sycl.hpp>
 #include <oneapi/mkl.hpp>
 
+#include <sycl/sycl.hpp>
 
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/range_accessors.hpp>
@@ -19,12 +18,13 @@
 #include <ginkgo/core/matrix/sellp.hpp>
 #include <ginkgo/core/matrix/sparsity_csr.hpp>
 
-
 #include "core/components/prefix_sum_kernels.hpp"
 #include "dpcpp/base/config.hpp"
 #include "dpcpp/base/dim3.dp.hpp"
 #include "dpcpp/base/helper.hpp"
+#include "dpcpp/base/math.hpp"
 #include "dpcpp/base/onemkl_bindings.hpp"
+#include "dpcpp/base/types.hpp"
 #include "dpcpp/components/cooperative_groups.dp.hpp"
 #include "dpcpp/components/reduction.dp.hpp"
 #include "dpcpp/components/thread_ids.dp.hpp"
@@ -103,12 +103,13 @@ void transpose(sycl::queue* queue, const matrix::Dense<ValueType>* orig,
 
     queue->submit([&](sycl::handler& cgh) {
         sycl::local_accessor<
-            uninitialized_array<ValueType, sg_size*(sg_size + 1)>, 0>
+            uninitialized_array<device_type<ValueType>, sg_size*(sg_size + 1)>,
+            0>
             space_acc_ct1(cgh);
         // Can not pass the member to device function directly
-        auto in = orig->get_const_values();
+        auto in = as_device_type(orig->get_const_values());
         auto in_stride = orig->get_stride();
-        auto out = trans->get_values();
+        auto out = as_device_type(trans->get_values());
         auto out_stride = trans->get_stride();
         cgh.parallel_for(
             sycl_nd_range(grid, block), [=](sycl::nd_item<3> item_ct1) {
@@ -220,17 +221,24 @@ void simple_apply(std::shared_ptr<const DefaultExecutor> exec,
                   matrix::Dense<ValueType>* c)
 {
     using namespace oneapi::mkl;
-    if (b->get_stride() != 0 && c->get_stride() != 0) {
-        if (a->get_size()[1] > 0) {
-            oneapi::mkl::blas::row_major::gemm(
-                *exec->get_queue(), transpose::nontrans, transpose::nontrans,
-                c->get_size()[0], c->get_size()[1], a->get_size()[1],
-                one<ValueType>(), a->get_const_values(), a->get_stride(),
-                b->get_const_values(), b->get_stride(), zero<ValueType>(),
-                c->get_values(), c->get_stride());
-        } else {
-            dense::fill(exec, c, zero<ValueType>());
+    if constexpr (onemkl::is_supported<ValueType>::value) {
+        if (b->get_stride() != 0 && c->get_stride() != 0) {
+            if (a->get_size()[1] > 0 && a->get_const_values() &&
+                b->get_const_values() && c->get_const_values()) {
+                oneapi::mkl::blas::row_major::gemm(
+                    *exec->get_queue(), transpose::nontrans,
+                    transpose::nontrans, c->get_size()[0], c->get_size()[1],
+                    a->get_size()[1], one<ValueType>(),
+                    as_device_type(a->get_const_values()), a->get_stride(),
+                    as_device_type(b->get_const_values()), b->get_stride(),
+                    zero<ValueType>(), as_device_type(c->get_values()),
+                    c->get_stride());
+            } else {
+                dense::fill(exec, c, zero<ValueType>());
+            }
         }
+    } else {
+        GKO_NOT_IMPLEMENTED;
     }
 }
 
@@ -244,19 +252,25 @@ void apply(std::shared_ptr<const DefaultExecutor> exec,
            const matrix::Dense<ValueType>* beta, matrix::Dense<ValueType>* c)
 {
     using namespace oneapi::mkl;
-    if (b->get_stride() != 0 && c->get_stride() != 0) {
-        if (a->get_size()[1] > 0) {
-            oneapi::mkl::blas::row_major::gemm(
-                *exec->get_queue(), transpose::nontrans, transpose::nontrans,
-                c->get_size()[0], c->get_size()[1], a->get_size()[1],
-                exec->copy_val_to_host(alpha->get_const_values()),
-                a->get_const_values(), a->get_stride(), b->get_const_values(),
-                b->get_stride(),
-                exec->copy_val_to_host(beta->get_const_values()),
-                c->get_values(), c->get_stride());
-        } else {
-            dense::scale(exec, beta, c);
+    if constexpr (onemkl::is_supported<ValueType>::value) {
+        if (b->get_stride() != 0 && c->get_stride() != 0) {
+            if (a->get_size()[1] > 0 && a->get_const_values() &&
+                b->get_const_values() && c->get_const_values()) {
+                oneapi::mkl::blas::row_major::gemm(
+                    *exec->get_queue(), transpose::nontrans,
+                    transpose::nontrans, c->get_size()[0], c->get_size()[1],
+                    a->get_size()[1],
+                    exec->copy_val_to_host(alpha->get_const_values()),
+                    as_device_type(a->get_const_values()), a->get_stride(),
+                    as_device_type(b->get_const_values()), b->get_stride(),
+                    exec->copy_val_to_host(beta->get_const_values()),
+                    as_device_type(c->get_values()), c->get_stride());
+            } else {
+                dense::scale(exec, beta, c);
+            }
         }
+    } else {
+        GKO_NOT_IMPLEMENTED;
     }
 }
 
@@ -271,12 +285,12 @@ void convert_to_coo(std::shared_ptr<const DefaultExecutor> exec,
 {
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
     const auto stride = source->get_stride();
 
     auto rows = result->get_row_idxs();
     auto cols = result->get_col_idxs();
-    auto vals = result->get_values();
+    auto vals = as_device_type(result->get_values());
 
     exec->get_queue()->submit([&](sycl::handler& cgh) {
         cgh.parallel_for(num_rows, [=](sycl::item<1> item) {
@@ -306,12 +320,12 @@ void convert_to_csr(std::shared_ptr<const DefaultExecutor> exec,
 {
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
     const auto stride = source->get_stride();
 
     const auto row_ptrs = result->get_const_row_ptrs();
     auto cols = result->get_col_idxs();
-    auto vals = result->get_values();
+    auto vals = as_device_type(result->get_values());
 
     exec->get_queue()->submit([&](sycl::handler& cgh) {
         cgh.parallel_for(num_rows, [=](sycl::item<1> item) {
@@ -341,11 +355,11 @@ void convert_to_ell(std::shared_ptr<const DefaultExecutor> exec,
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
     const auto max_nnz_per_row = result->get_num_stored_elements_per_row();
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
     const auto in_stride = source->get_stride();
 
     auto cols = result->get_col_idxs();
-    auto vals = result->get_values();
+    auto vals = as_device_type(result->get_values());
     const auto stride = result->get_stride();
 
     exec->get_queue()->submit([&](sycl::handler& cgh) {
@@ -362,7 +376,7 @@ void convert_to_ell(std::shared_ptr<const DefaultExecutor> exec,
             }
             for (; col_idx < max_nnz_per_row; col_idx++) {
                 cols[col_idx * stride + row] = invalid_index<IndexType>();
-                vals[col_idx * stride + row] = zero<ValueType>();
+                vals[col_idx * stride + row] = zero<device_type<ValueType>>();
             }
         });
     });
@@ -401,14 +415,14 @@ void convert_to_hybrid(std::shared_ptr<const DefaultExecutor> exec,
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
     const auto ell_lim = result->get_ell_num_stored_elements_per_row();
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
     const auto in_stride = source->get_stride();
     const auto ell_stride = result->get_ell_stride();
     auto ell_cols = result->get_ell_col_idxs();
-    auto ell_vals = result->get_ell_values();
+    auto ell_vals = as_device_type(result->get_ell_values());
     auto coo_rows = result->get_coo_row_idxs();
     auto coo_cols = result->get_coo_col_idxs();
-    auto coo_vals = result->get_coo_values();
+    auto coo_vals = as_device_type(result->get_coo_values());
 
     exec->get_queue()->submit([&](sycl::handler& cgh) {
         cgh.parallel_for(num_rows, [=](sycl::item<1> item) {
@@ -426,7 +440,7 @@ void convert_to_hybrid(std::shared_ptr<const DefaultExecutor> exec,
                 }
             }
             for (; ell_count < ell_lim; ell_count++) {
-                ell_vals[ell_idx] = zero<ValueType>();
+                ell_vals[ell_idx] = zero<device_type<ValueType>>();
                 ell_cols[ell_idx] = invalid_index<IndexType>();
                 ell_idx += ell_stride;
             }
@@ -456,11 +470,11 @@ void convert_to_sellp(std::shared_ptr<const DefaultExecutor> exec,
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
     const auto stride = source->get_stride();
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
 
     const auto slice_sets = result->get_const_slice_sets();
     const auto slice_size = result->get_slice_size();
-    auto vals = result->get_values();
+    auto vals = as_device_type(result->get_values());
     auto col_idxs = result->get_col_idxs();
 
     exec->get_queue()->submit([&](sycl::handler& cgh) {
@@ -481,7 +495,7 @@ void convert_to_sellp(std::shared_ptr<const DefaultExecutor> exec,
             }
             for (; out_idx < slice_end; out_idx += slice_size) {
                 col_idxs[out_idx] = invalid_index<IndexType>();
-                vals[out_idx] = zero<ValueType>();
+                vals[out_idx] = zero<device_type<ValueType>>();
             }
         });
     });
@@ -498,7 +512,7 @@ void convert_to_sparsity_csr(std::shared_ptr<const DefaultExecutor> exec,
 {
     const auto num_rows = result->get_size()[0];
     const auto num_cols = result->get_size()[1];
-    const auto in_vals = source->get_const_values();
+    const auto in_vals = as_device_type(source->get_const_values());
     const auto stride = source->get_stride();
 
     const auto row_ptrs = result->get_const_row_ptrs();
@@ -563,9 +577,10 @@ void conj_transpose(std::shared_ptr<const DefaultExecutor> exec,
     const auto sg_size = DCFG_1D::decode<1>(cfg);
     dim3 grid(ceildiv(size[1], sg_size), ceildiv(size[0], sg_size));
     dim3 block(sg_size, sg_size);
-    kernel::conj_transpose_call(cfg, grid, block, 0, queue, size[0], size[1],
-                                orig->get_const_values(), orig->get_stride(),
-                                trans->get_values(), trans->get_stride());
+    kernel::conj_transpose_call(
+        cfg, grid, block, 0, queue, size[0], size[1],
+        as_device_type(orig->get_const_values()), orig->get_stride(),
+        as_device_type(trans->get_values()), trans->get_stride());
 }
 
 GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_CONJ_TRANSPOSE_KERNEL);
