@@ -2,22 +2,17 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/preconditioner/jacobi.hpp>
-
-
 #include <initializer_list>
 #include <random>
 #include <vector>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/types.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
-
+#include <ginkgo/core/preconditioner/jacobi.hpp>
 
 #include "core/test/utils.hpp"
 #include "core/test/utils/unsort_matrix.hpp"
@@ -28,6 +23,8 @@
 namespace {
 
 
+// We keep some distribution with value_type to make the test with
+// GINKGO_DPCPP_SINGLE_MODE still work.
 class Jacobi : public ::testing::Test {
 protected:
     using index_type = int32_t;
@@ -910,4 +907,27 @@ TEST_F(
 }
 
 
+TEST_F(Jacobi, ScalarJacobiHandleZero)
+{
+    auto mtx = gko::share(
+        gko::initialize<Vec>({{0, 0, 0}, {0, 2, 0}, {0, 0, 0}}, ref));
+    auto b = gko::initialize<Vec>({1, 2, 3}, ref);
+    auto x = Vec::create(ref, gko::dim<2>(3, 1));
+    auto d_b = b->clone(dpcpp);
+    auto d_x = x->clone(dpcpp);
+    auto d_mtx = gko::share(mtx->clone(dpcpp));
+
+    auto jacobi = Bj::build().with_max_block_size(1u).on(ref)->generate(mtx);
+    // Must generate from scratch because the clone copies the inverted
+    // information.
+    auto d_jacobi =
+        Bj::build().with_max_block_size(1u).on(dpcpp)->generate(d_mtx);
+
+    // Jacobi uses 1 as the result when diagonal value is zero.
+    jacobi->apply(b, x);
+    d_jacobi->apply(d_b, d_x);
+    GKO_ASSERT_MTX_NEAR(d_x, x, 0.0);
+}
+
+
 }  // namespace
