@@ -4,7 +4,6 @@
 
 #include "core/matrix/fft_kernels.hpp"
 
-
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
@@ -35,7 +34,7 @@ void fft(std::shared_ptr<const DefaultExecutor> exec,
          matrix::Dense<std::complex<ValueType>>* x, bool inverse,
          array<char>& buffer) GKO_NOT_IMPLEMENTED;
 
-GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE(GKO_DECLARE_FFT_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE_BASE(GKO_DECLARE_FFT_KERNEL);
 
 
 template <typename ValueType>
@@ -45,7 +44,7 @@ void fft2(std::shared_ptr<const DefaultExecutor> exec,
           size_type size2, bool inverse,
           array<char>& buffer) GKO_NOT_IMPLEMENTED;
 
-GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE(GKO_DECLARE_FFT2_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE_BASE(GKO_DECLARE_FFT2_KERNEL);
 
 
 template <typename ValueType>
@@ -55,7 +54,7 @@ void fft3(std::shared_ptr<const DefaultExecutor> exec,
           size_type size2, size_type size3, bool inverse,
           array<char>& buffer) GKO_NOT_IMPLEMENTED;
 
-GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE(GKO_DECLARE_FFT3_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_TYPE_BASE(GKO_DECLARE_FFT3_KERNEL);
 
 
 }  // namespace fft
