@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -21,6 +21,7 @@
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/base/math.hpp>
 
+#include "common/cuda_hip/base/bf16_alias.hpp"
 #include "common/cuda_hip/base/types.hpp"
 #include "hip/test/utils.hip.hpp"
 
@@ -131,6 +132,25 @@ TEST_F(IsFinite, HalfComplex)
 #endif  // GINKGO_ENABLE_HALF
 
 
+#if GINKGO_ENABLE_BFLOAT16
+
+
+TEST_F(IsFinite, Bfloat16)
+{
+    ASSERT_TRUE(test_real_is_finite_kernel<gko::vendor_bf16>());
+}
+
+
+TEST_F(IsFinite, Bfloat16Complex)
+{
+    ASSERT_TRUE(
+        test_complex_is_finite_kernel<thrust::complex<gko::vendor_bf16>>());
+}
+
+
+#endif  // GINKGO_ENABLE_BFLOAT16
+
+
 TEST_F(IsFinite, FloatComplex)
 {
     ASSERT_TRUE(test_complex_is_finite_kernel<thrust::complex<float>>());
