@@ -2,20 +2,16 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include "hip/components/sorting.hip.hpp"
-
+#include "common/cuda_hip/components/sorting.hpp"
 
 #include <memory>
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/executor.hpp>
 
-
 #include "hip/test/utils.hip.hpp"
 
 
