@@ -1,4 +1,4 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
@@ -146,7 +146,8 @@ TYPED_TEST(Fbcsr, SpmvIsEquivalentToRefSorted)
         this->ref, gko::dim<2>(this->rsorted_ref->get_size()[0], 1));
     auto prod_hip = Dense::create(this->exec, prod_ref->get_size());
 
-    if (std::is_same<value_type, gko::half>::value) {
+    if (std::is_same<value_type, gko::float16>::value ||
+        std::is_same<value_type, gko::bfloat16>::value) {
         ASSERT_THROW(rand_hip->apply(x_hip, prod_hip), gko::NotImplemented);
     } else {
         rand_hip->apply(x_hip, prod_hip);
@@ -174,7 +175,8 @@ TYPED_TEST(Fbcsr, SpmvMultiIsEquivalentToRefSorted)
         this->ref, gko::dim<2>(this->rsorted_ref->get_size()[0], 3));
     auto prod_hip = Dense::create(this->exec, prod_ref->get_size());
 
-    if (std::is_same<value_type, gko::half>::value) {
+    if (std::is_same<value_type, gko::float16>::value ||
+        std::is_same<value_type, gko::bfloat16>::value) {
         ASSERT_THROW(rand_hip->apply(x_hip, prod_hip), gko::NotImplemented);
     } else {
         rand_hip->apply(x_hip, prod_hip);
@@ -214,7 +216,8 @@ TYPED_TEST(Fbcsr, AdvancedSpmvIsEquivalentToRefSorted)
     auto beta = Dense::create(this->exec);
     beta->copy_from(beta_ref);
 
-    if (std::is_same<value_type, gko::half>::value) {
+    if (std::is_same<value_type, gko::float16>::value ||
+        std::is_same<value_type, gko::bfloat16>::value) {
         ASSERT_THROW(rand_hip->apply(alpha, x_hip, beta, prod_hip),
                      gko::NotImplemented);
     } else {
@@ -255,7 +258,8 @@ TYPED_TEST(Fbcsr, AdvancedSpmvMultiIsEquivalentToRefSorted)
     auto beta = Dense::create(this->exec);
     beta->copy_from(beta_ref);
 
-    if (std::is_same<value_type, gko::half>::value) {
+    if (std::is_same<value_type, gko::float16>::value ||
+        std::is_same<value_type, gko::bfloat16>::value) {
         ASSERT_THROW(rand_hip->apply(alpha, x_hip, beta, prod_hip),
                      gko::NotImplemented);
     } else {
