@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GKO_PUBLIC_CORE_PRECONDITIONER_JACOBI_HPP_
 #define GKO_PUBLIC_CORE_PRECONDITIONER_JACOBI_HPP_
@@ -36,6 +8,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/lin_op.hpp>
+#include <ginkgo/core/config/config.hpp>
+#include <ginkgo/core/config/registry.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
@@ -288,7 +262,7 @@ public:
      */
     size_type get_num_stored_elements() const noexcept
     {
-        return blocks_.get_num_elems();
+        return blocks_.get_size();
     }
 
     void convert_to(matrix::Dense<value_type>* result) const override;
@@ -397,6 +371,19 @@ public:
         gko::array<index_type> GKO_FACTORY_PARAMETER_VECTOR(block_pointers,
                                                             nullptr);
 
+        /**
+         * Use L1 Jacboi, which is introduced in the paper A. H. Baker et al.
+         * "Multigrid smoothers for ultraparallel computing." This paper
+         * discusses this type of smoother with the following matrix property.
+         * $ A_{ii} \geq \theta \sum_{j \in \text{off diagonal block}} |A_{ij}|,
+         * \theta \geq 0 $
+         * If it is true, it generates the preconditioner on A +
+         * Diag(sum_{k in off-diagonal block of i} |A_ik|) instead of A. We
+         * aggregate the absolute value of the entries with the same row in the
+         * off-diagonal block into the diagonal value.
+         */
+        bool GKO_FACTORY_PARAMETER_SCALAR(aggregate_l1, false);
+
     private:
         // See documentation of storage_optimization parameter for details about
         // this class
@@ -407,13 +394,13 @@ public:
 
             storage_optimization_type(
                 const array<precision_reduction>& block_wise_opt)
-                : is_block_wise{block_wise_opt.get_num_elems() > 0},
+                : is_block_wise{block_wise_opt.get_size() > 0},
                   block_wise{block_wise_opt}
             {}
 
             storage_optimization_type(
                 array<precision_reduction>&& block_wise_opt)
-                : is_block_wise{block_wise_opt.get_num_elems() > 0},
+                : is_block_wise{block_wise_opt.get_size() > 0},
                   block_wise{std::move(block_wise_opt)}
             {}
 
@@ -527,6 +514,27 @@ public:
     GKO_ENABLE_LIN_OP_FACTORY(Jacobi, parameters, Factory);
     GKO_ENABLE_BUILD_METHOD(Factory);
 
+    /**
+     * Create the parameters from the property_tree.
+     * Because this is directly tied to the specific type, the value/index type
+     * settings within config are ignored and type_descriptor is only used
+     * for children configs.
+     *
+     * @param config  the property tree for setting
+     * @param context  the registry
+     * @param td_for_child  the type descriptor for children configs. The
+     *                      default uses the value/index type of this class.
+     *
+     * @return parameters
+     *
+     * @note Jacobi does not support block_pointers and storage_optimization
+     *       array.
+     */
+    static parameters_type parse(
+        const config::pnode& config, const config::registry& context,
+        const config::type_descriptor& td_for_child =
+            config::make_type_descriptor<ValueType, IndexType>());
+
 protected:
     /**
      * Creates an empty Jacobi preconditioner.
@@ -557,10 +565,10 @@ protected:
           parameters_{factory->get_parameters()},
           storage_scheme_{this->compute_storage_scheme(
               parameters_.max_block_size, parameters_.max_block_stride)},
-          num_blocks_{parameters_.block_pointers.get_num_elems() - 1},
+          num_blocks_{parameters_.block_pointers.get_size() - 1},
           blocks_(factory->get_executor(),
                   storage_scheme_.compute_storage_space(
-                      parameters_.block_pointers.get_num_elems() - 1)),
+                      parameters_.block_pointers.get_size() - 1)),
           conditioning_(factory->get_executor())
     {
         parameters_.block_pointers.set_executor(this->get_executor());
