@@ -4,16 +4,13 @@
 
 #include "core/solver/cb_gmres_kernels.hpp"
 
-
 #include <omp.h>
 
-
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/types.hpp>
 #include <ginkgo/core/solver/cb_gmres.hpp>
 #include <ginkgo/core/stop/stopping_status.hpp>
 
-
 #include "common/unified/base/kernel_launch_reduction.hpp"
 #include "core/solver/cb_gmres_accessor.hpp"
 
@@ -333,7 +330,8 @@ void initialize(std::shared_ptr<const OmpExecutor> exec,
     }
 }
 
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_CB_GMRES_INITIALIZE_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE_BASE(
+    GKO_DECLARE_CB_GMRES_INITIALIZE_KERNEL);
 
 
 template <typename ValueType, typename Accessor3d>
