@@ -1,11 +1,11 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/components/fill_array_kernels.hpp"
 
-
 #include <numeric>
+#include <type_traits>
 
 
 namespace gko {
@@ -23,6 +23,11 @@ void fill_array(std::shared_ptr<const DefaultExecutor> exec, ValueType* array,
 
 GKO_INSTANTIATE_FOR_EACH_TEMPLATE_TYPE(GKO_DECLARE_FILL_ARRAY_KERNEL);
 template GKO_DECLARE_FILL_ARRAY_KERNEL(bool);
+template GKO_DECLARE_FILL_ARRAY_KERNEL(uint16);
+template GKO_DECLARE_FILL_ARRAY_KERNEL(uint32);
+#ifndef GKO_SIZE_T_IS_UINT64_T
+template GKO_DECLARE_FILL_ARRAY_KERNEL(uint64);
+#endif
 
 
 template <typename ValueType>
