@@ -1,25 +1,21 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/factorization/par_ilut.hpp>
-
+#include "core/factorization/par_ilut_kernels.hpp"
 
 #include <algorithm>
 #include <memory>
 #include <vector>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/executor.hpp>
+#include <ginkgo/core/factorization/par_ilut.hpp>
 #include <ginkgo/core/matrix/coo.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 
-
-#include "core/factorization/par_ilut_kernels.hpp"
 #include "core/test/utils.hpp"
 
 
@@ -58,6 +54,7 @@ protected:
     using ComplexCsr =
         gko::matrix::Csr<std::complex<gko::remove_complex<value_type>>,
                          index_type>;
+    using complex_value_type = std::complex<gko::remove_complex<value_type>>;
 
     ParIlut()
         : ref(gko::ReferenceExecutor::create()),
@@ -79,16 +76,24 @@ protected:
                                                   {0., -3., 0., 1.}},
                                                  ref)),
           mtx1_complex(gko::initialize<ComplexCsr>(
-              {{{.1, 0.}, {0., 0.}, {0., 0.}, {0., 0.}},
-               {{-1., .1}, {.1, -1.}, {0., 0.}, {0., 0.}},
-               {{-1., 1.}, {-2., .2}, {-1., -.3}, {0., 0.}},
-               {{1., -2.}, {-3., -.1}, {-1., .1}, {.1, 2.}}},
+              {{complex_value_type{.1, 0.}, complex_value_type{0., 0.},
+                complex_value_type{0., 0.}, complex_value_type{0., 0.}},
+               {complex_value_type{-1., .1}, complex_value_type{.1, -1.},
+                complex_value_type{0., 0.}, complex_value_type{0., 0.}},
+               {complex_value_type{-1., 1.}, complex_value_type{-2., .2},
+                complex_value_type{-1., -.3}, complex_value_type{0., 0.}},
+               {complex_value_type{1., -2.}, complex_value_type{-3., -.1},
+                complex_value_type{-1., .1}, complex_value_type{.1, 2.}}},
               ref)),
           mtx1_expect_complex_thrm(gko::initialize<ComplexCsr>(
-              {{{.1, 0.}, {0., 0.}, {0., 0.}, {0., 0.}},
-               {{0., 0.}, {.1, -1.}, {0., 0.}, {0., 0.}},
-               {{-1., 1.}, {-2., .2}, {-1., -.3}, {0., 0.}},
-               {{1., -2.}, {-3., -.1}, {0., 0.}, {.1, 2.}}},
+              {{complex_value_type{.1, 0.}, complex_value_type{0., 0.},
+                complex_value_type{0., 0.}, complex_value_type{0., 0.}},
+               {complex_value_type{0., 0.}, complex_value_type{.1, -1.},
+                complex_value_type{0., 0.}, complex_value_type{0., 0.}},
+               {complex_value_type{-1., 1.}, complex_value_type{-2., .2},
+                complex_value_type{-1., -.3}, complex_value_type{0., 0.}},
+               {complex_value_type{1., -2.}, complex_value_type{-3., -.1},
+                complex_value_type{0., 0.}, complex_value_type{.1, 2.}}},
               ref)),
           identity(gko::initialize<Csr>(
               {{1., 0., 0.}, {0., 1., 0.}, {0., 0., 1.}}, ref)),
@@ -380,14 +385,14 @@ TYPED_TEST(ParIlut, KernelComplexThresholdFilterNoneUpper)
 
 TYPED_TEST(ParIlut, KernelComplexThresholdFilterSomeAtThresholdLower)
 {
-    this->test_filter(this->mtx1_complex, 1.01, this->mtx1_expect_complex_thrm,
+    this->test_filter(this->mtx1_complex, 1.02, this->mtx1_expect_complex_thrm,
                       true);
 }
 
 
 TYPED_TEST(ParIlut, KernelComplexThresholdFilterSomeAtThresholdUpper)
 {
-    this->test_filter(this->mtx1_complex, 1.01, this->mtx1_expect_complex_thrm,
+    this->test_filter(this->mtx1_complex, 1.02, this->mtx1_expect_complex_thrm,
                       false);
 }
 
