@@ -2,25 +2,21 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/matrix/diagonal.hpp>
-
+#include "core/matrix/diagonal_kernels.hpp"
 
 #include <algorithm>
 #include <complex>
 #include <memory>
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
+#include <ginkgo/core/matrix/diagonal.hpp>
 
-
-#include "core/matrix/diagonal_kernels.hpp"
 #include "core/test/utils.hpp"
 
 
@@ -89,7 +85,7 @@ TYPED_TEST_SUITE(Diagonal, gko::test::ValueTypes, TypenameNameGenerator);
 TYPED_TEST(Diagonal, ConvertsToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Diagonal = typename TestFixture::Diag;
     using OtherDiagonal = gko::matrix::Diagonal<OtherType>;
     auto tmp = OtherDiagonal::create(this->exec);
@@ -97,7 +93,9 @@ TYPED_TEST(Diagonal, ConvertsToPrecision)
     // If OtherType is more precise: 0, otherwise r
     auto residual = r<OtherType>::value < r<ValueType>::value
                         ? gko::remove_complex<ValueType>{0}
-                        : gko::remove_complex<ValueType>{r<OtherType>::value};
+                        : gko::remove_complex<ValueType>{
+                              static_cast<gko::remove_complex<ValueType>>(
+                                  r<OtherType>::value)};
 
     this->diag1->convert_to(tmp);
     tmp->convert_to(res);
@@ -109,15 +107,16 @@ TYPED_TEST(Diagonal, ConvertsToPrecision)
 TYPED_TEST(Diagonal, MovesToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Diagonal = typename TestFixture::Diag;
     using OtherDiagonal = gko::matrix::Diagonal<OtherType>;
     auto tmp = OtherDiagonal::create(this->exec);
     auto res = Diagonal::create(this->exec);
     // If OtherType is more precise: 0, otherwise r
-    auto residual = r<OtherType>::value < r<ValueType>::value
-                        ? gko::remove_complex<ValueType>{0}
-                        : gko::remove_complex<ValueType>{r<OtherType>::value};
+    auto residual =
+        r<OtherType>::value < r<ValueType>::value
+            ? gko::remove_complex<ValueType>{0}
+            : static_cast<gko::remove_complex<ValueType>>(r<OtherType>::value);
 
     this->diag1->move_to(tmp);
     tmp->move_to(res);
