@@ -1,23 +1,18 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/matrix/ell.hpp>
-
-
 #include <memory>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
-
+#include <ginkgo/core/matrix/ell.hpp>
 
 #include "core/test/utils.hpp"
 
@@ -249,6 +244,21 @@ TYPED_TEST(Ell, AppliesLinearCombinationToDenseVector)
 }
 
 
+TYPED_TEST(Ell, AppliesLinearCombinationToDenseVectorWithZeroBetaNaN)
+{
+    using T = typename TestFixture::value_type;
+    using Vec = typename TestFixture::Vec;
+    auto alpha = gko::initialize<Vec>({-1.0}, this->exec);
+    auto beta = gko::initialize<Vec>({0.0}, this->exec);
+    auto x = gko::initialize<Vec>({2.0, 1.0, 4.0}, this->exec);
+    auto y = gko::initialize<Vec>({gko::nan<T>(), gko::nan<T>()}, this->exec);
+
+    this->mtx1->apply(alpha, x, beta, y);
+
+    GKO_ASSERT_MTX_NEAR(y, l({-13.0, -5.0}), 0.0);
+}
+
+
 TYPED_TEST(Ell, MixedAppliesLinearCombinationToDenseVector1)
 {
     // Both vectors have the same value type which differs from the matrix
@@ -448,7 +458,7 @@ TYPED_TEST(Ell, ConvertsToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
     using IndexType = typename TestFixture::index_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Ell = typename TestFixture::Mtx;
     using OtherEll = gko::matrix::Ell<OtherType, IndexType>;
     auto tmp = OtherEll::create(this->exec);
@@ -456,7 +466,9 @@ TYPED_TEST(Ell, ConvertsToPrecision)
     // If OtherType is more precise: 0, otherwise r
     auto residual = r<OtherType>::value < r<ValueType>::value
                         ? gko::remove_complex<ValueType>{0}
-                        : gko::remove_complex<ValueType>{r<OtherType>::value};
+                        : gko::remove_complex<ValueType>{
+                              static_cast<gko::remove_complex<ValueType>>(
+                                  r<OtherType>::value)};
 
     this->mtx1->convert_to(tmp);
     tmp->convert_to(res);
@@ -469,15 +481,16 @@ TYPED_TEST(Ell, MovesToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
     using IndexType = typename TestFixture::index_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Ell = typename TestFixture::Mtx;
     using OtherEll = gko::matrix::Ell<OtherType, IndexType>;
     auto tmp = OtherEll::create(this->exec);
     auto res = Ell::create(this->exec);
     // If OtherType is more precise: 0, otherwise r
-    auto residual = r<OtherType>::value < r<ValueType>::value
-                        ? gko::remove_complex<ValueType>{0}
-                        : gko::remove_complex<ValueType>{r<OtherType>::value};
+    auto residual =
+        r<OtherType>::value < r<ValueType>::value
+            ? gko::remove_complex<ValueType>{0}
+            : static_cast<gko::remove_complex<ValueType>>(r<OtherType>::value);
 
     this->mtx1->move_to(tmp);
     tmp->move_to(res);
@@ -737,7 +750,7 @@ TYPED_TEST(Ell, ConvertsEmptyToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
     using IndexType = typename TestFixture::index_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Ell = typename TestFixture::Mtx;
     using OtherEll = gko::matrix::Ell<OtherType, IndexType>;
     auto empty = Ell::create(this->exec);
@@ -754,7 +767,7 @@ TYPED_TEST(Ell, MovesEmptyToPrecision)
 {
     using ValueType = typename TestFixture::value_type;
     using IndexType = typename TestFixture::index_type;
-    using OtherType = typename gko::next_precision<ValueType>;
+    using OtherType = gko::next_precision<ValueType>;
     using Ell = typename TestFixture::Mtx;
     using OtherEll = gko::matrix::Ell<OtherType, IndexType>;
     auto empty = Ell::create(this->exec);
