@@ -2,15 +2,10 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/multigrid/fixed_coarsening.hpp>
-
-
 #include <memory>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/base/math.hpp>
@@ -20,12 +15,12 @@
 #include <ginkgo/core/matrix/diagonal.hpp>
 #include <ginkgo/core/matrix/row_gatherer.hpp>
 #include <ginkgo/core/matrix/sparsity_csr.hpp>
+#include <ginkgo/core/multigrid/fixed_coarsening.hpp>
 #include <ginkgo/core/stop/combined.hpp>
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 #include <ginkgo/core/stop/time.hpp>
 
-
 #include "core/test/utils.hpp"
 
 
@@ -48,18 +43,19 @@ protected:
     using real_type = gko::remove_complex<value_type>;
     FixedCoarsening()
         : exec(gko::ReferenceExecutor::create()),
+          mtx(Mtx::create(exec, gko::dim<2>(5, 5), 15,
+                          std::make_shared<typename Mtx::classical>())),
+          coarse(Mtx::create(exec, gko::dim<2>(3, 3), 5,
+                             std::make_shared<typename Mtx::classical>())),
           coarse_rows(exec, {0, 2, 3}),
-          fixed_coarsening_factory(MgLevel::build()
-                                       .with_coarse_rows(coarse_rows)
-                                       .with_skip_sorting(true)
-                                       .on(exec)),
+          gen_coarse_rows(exec, 5),
+          coarse_b(gko::initialize<Vec>(
+              {I<VT>({2.0, -1.0}), I<VT>({3.0, 1.0}), I<VT>({0.0, -1.0})},
+              exec)),
           fine_b(gko::initialize<Vec>(
               {I<VT>({2.0, -1.0}), I<VT>({-1.0, 2.0}), I<VT>({0.0, -1.0}),
                I<VT>({3.0, -2.0}), I<VT>({-2.0, 1.0})},
               exec)),
-          coarse_b(gko::initialize<Vec>(
-              {I<VT>({2.0, -1.0}), I<VT>({3.0, 1.0}), I<VT>({0.0, -1.0})},
-              exec)),
           restrict_ans(gko::initialize<Vec>(
               {I<VT>({2.0, -1.0}), I<VT>({0.0, -1.0}), I<VT>({3.0, -2.0})},
               exec)),
@@ -71,11 +67,10 @@ protected:
               {I<VT>({-2.0, -1.0}), I<VT>({1.0, -1.0}), I<VT>({-1.0, -1.0}),
                I<VT>({0.0, 0.0}), I<VT>({0.0, 2.0})},
               exec)),
-          mtx(Mtx::create(exec, gko::dim<2>(5, 5), 15,
-                          std::make_shared<typename Mtx::classical>())),
-          coarse(Mtx::create(exec, gko::dim<2>(3, 3), 5,
-                             std::make_shared<typename Mtx::classical>())),
-          gen_coarse_rows(exec, 5)
+          fixed_coarsening_factory(MgLevel::build()
+                                       .with_coarse_rows(coarse_rows)
+                                       .with_skip_sorting(true)
+                                       .on(exec))
     {
         this->create_mtx(mtx.get(), &gen_coarse_rows, coarse.get());
         mg_level = fixed_coarsening_factory->generate(mtx);
