@@ -1,30 +1,26 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/preconditioner/isai.hpp>
-
+#include "core/preconditioner/isai_kernels.hpp"
 
 #include <algorithm>
 #include <fstream>
 #include <memory>
 #include <type_traits>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/mtx_io.hpp>
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/preconditioner/ilu.hpp>
+#include <ginkgo/core/preconditioner/isai.hpp>
 #include <ginkgo/core/preconditioner/jacobi.hpp>
 #include <ginkgo/core/solver/gmres.hpp>
 
-
 #include "core/base/utils.hpp"
-#include "core/preconditioner/isai_kernels.hpp"
 #include "core/test/utils.hpp"
 #include "matrices/config.hpp"
 
@@ -190,8 +186,20 @@ protected:
     {
         lower_isai_factory = LowerIsai::build().on(exec);
         upper_isai_factory = UpperIsai::build().on(exec);
-        general_isai_factory = GeneralIsai::build().on(exec);
-        spd_isai_factory = SpdIsai::build().on(exec);
+        if (std::is_same_v<gko::remove_complex<value_type>, gko::float16>) {
+            general_isai_factory =
+                GeneralIsai::build()
+                    .with_excess_solver_reduction(
+                        gko::remove_complex<value_type>{1e-3})
+                    .on(exec);
+            spd_isai_factory = SpdIsai::build()
+                                   .with_excess_solver_reduction(
+                                       gko::remove_complex<value_type>{1e-3})
+                                   .on(exec);
+        } else {
+            general_isai_factory = GeneralIsai::build().on(exec);
+            spd_isai_factory = SpdIsai::build().on(exec);
+        }
         a_dense->convert_to(a_csr);
         a_dense_inv->convert_to(a_csr_inv);
         l_dense->convert_to(l_csr);
@@ -314,9 +322,30 @@ protected:
     std::shared_ptr<Csr> spd_sparse_inv;
 };
 
+#ifdef __NVCOMPILER
+
+
+// Due to NVHPC compilation limitation, we need to split it to two files.
+#ifdef NVHPC_FLOAT16
+using Float16IndexTypes = gko::test::cartesian_type_product_t<
+    ::testing::Types<gko::float16, std::complex<gko::float16>>,
+    gko::test::IndexTypes>;
+TYPED_TEST_SUITE(Isai, Float16IndexTypes, PairTypenameNameGenerator);
+#else
+TYPED_TEST_SUITE(Isai, gko::test::ValueIndexTypesBase,
+                 PairTypenameNameGenerator);
+#endif
+
+
+#else
+
+
 TYPED_TEST_SUITE(Isai, gko::test::ValueIndexTypes, PairTypenameNameGenerator);
 
 
+#endif
+
+
 TYPED_TEST(Isai, KernelGenerateA)
 {
     using Csr = typename TestFixture::Csr;
@@ -1025,6 +1054,9 @@ TYPED_TEST(Isai, ReturnsCorrectInverseALongrowWithExcessSolver)
 {
     using value_type = typename TestFixture::value_type;
     using GeneralIsai = typename TestFixture::GeneralIsai;
+    // When using the other precision, we already need to drastically reduce the
+    // precision, so it is hard to work with half.
+    SKIP_IF_HALF(value_type);
     auto general_isai_factory =
         GeneralIsai::build()
             .with_excess_solver_factory(this->excess_solver_factory)
@@ -1072,6 +1104,9 @@ TYPED_TEST(Isai, ReturnsCorrectInverseLLongrowWithExcessSolver)
     using Csr = typename TestFixture::Csr;
     using LowerIsai = typename TestFixture::LowerIsai;
     using value_type = typename TestFixture::value_type;
+    // When using the other precision, we already need to drastically reduce the
+    // precision, so it is hard to work with half.
+    SKIP_IF_HALF(value_type);
     auto lower_isai_factory =
         LowerIsai::build()
             .with_excess_solver_factory(this->excess_solver_factory)
@@ -1119,6 +1154,9 @@ TYPED_TEST(Isai, ReturnsCorrectInverseULongrowWithExcessSolver)
     using Csr = typename TestFixture::Csr;
     using UpperIsai = typename TestFixture::UpperIsai;
     using value_type = typename TestFixture::value_type;
+    // When using the other precision, we already need to drastically reduce the
+    // precision, so it is hard to work with half.
+    SKIP_IF_HALF(value_type);
     auto upper_isai_factory =
         UpperIsai::build()
             .with_excess_solver_factory(this->excess_solver_factory)
@@ -1232,8 +1270,8 @@ TYPED_TEST(Isai, ReturnsCorrectInverseSpdLongrow)
     // need to reduce precision due to spd ISAI using GMRES instead of
     // direct solve
     GKO_ASSERT_MTX_NEAR(lower, this->spd_csr_longrow_inv,
-                        10 * r<value_type>::value);
-    GKO_ASSERT_MTX_NEAR(lower_t, expected_transpose, 10 * r<value_type>::value);
+                        30 * r<value_type>::value);
+    GKO_ASSERT_MTX_NEAR(lower_t, expected_transpose, 30 * r<value_type>::value);
 }
 
 
@@ -1242,6 +1280,9 @@ TYPED_TEST(Isai, ReturnsCorrectInverseSpdLongrowWithExcessSolver)
     using Csr = typename TestFixture::Csr;
     using SpdIsai = typename TestFixture::SpdIsai;
     using value_type = typename TestFixture::value_type;
+    // When using the other precision, we already need to drastically reduce the
+    // precision, so it is hard to work with half.
+    SKIP_IF_HALF(value_type);
     const auto expected_transpose =
         gko::as<Csr>(this->spd_csr_longrow_inv->transpose());
     auto spd_isai_factory =
