@@ -2,28 +2,24 @@
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/solver/gcr.hpp>
-
+#include "core/solver/gcr_kernels.hpp"
 
 #include <algorithm>
 #include <limits>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/preconditioner/jacobi.hpp>
+#include <ginkgo/core/solver/gcr.hpp>
 #include <ginkgo/core/stop/combined.hpp>
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 #include <ginkgo/core/stop/time.hpp>
 
-
-#include "core/solver/gcr_kernels.hpp"
 #include "core/test/utils.hpp"
 
 
@@ -44,6 +40,21 @@ protected:
           non_stopped{},
           mtx(gko::initialize<Mtx>(
               {{1.0, 2.0, 3.0}, {3.0, 2.0, -1.0}, {0.0, -1.0, 2}}, exec)),
+          mtx_medium(
+              gko::initialize<Mtx>({{-86.40, 153.30, -108.90, 8.60, -61.60},
+                                    {7.70, -77.00, 3.30, -149.20, 74.80},
+                                    {-121.40, 37.10, 55.30, -74.20, -19.20},
+                                    {-111.40, -22.60, 110.10, -106.20, 88.90},
+                                    {-0.70, 111.70, 154.40, 235.00, -76.50}},
+                                   exec)),
+          mtx_big(gko::initialize<Mtx>(
+              {{2295.7, -764.8, 1166.5, 428.9, 291.7, -774.5},
+               {2752.6, -1127.7, 1212.8, -299.1, 987.7, 786.8},
+               {138.3, 78.2, 485.5, -899.9, 392.9, 1408.9},
+               {-1907.1, 2106.6, 1026.0, 634.7, 194.6, -534.1},
+               {-365.0, -715.8, 870.7, 67.5, 279.8, 1927.8},
+               {-848.1, -280.5, -381.8, -187.1, 51.2, -176.2}},
+              exec)),
           gcr_factory(Solver::build()
                           .with_criteria(
                               gko::stop::Iteration::build().with_max_iters(4u),
@@ -53,14 +64,6 @@ protected:
                                   .with_reduction_factor(r<value_type>::value))
                           .with_krylov_dim(3u)
                           .on(exec)),
-          mtx_big(gko::initialize<Mtx>(
-              {{2295.7, -764.8, 1166.5, 428.9, 291.7, -774.5},
-               {2752.6, -1127.7, 1212.8, -299.1, 987.7, 786.8},
-               {138.3, 78.2, 485.5, -899.9, 392.9, 1408.9},
-               {-1907.1, 2106.6, 1026.0, 634.7, 194.6, -534.1},
-               {-365.0, -715.8, 870.7, 67.5, 279.8, 1927.8},
-               {-848.1, -280.5, -381.8, -187.1, 51.2, -176.2}},
-              exec)),
           gcr_factory_big(
               Solver::build()
                   .with_criteria(
@@ -74,14 +77,7 @@ protected:
                       gko::stop::Iteration::build().with_max_iters(100u),
                       gko::stop::ImplicitResidualNorm<value_type>::build()
                           .with_reduction_factor(r<value_type>::value))
-                  .on(exec)),
-          mtx_medium(
-              gko::initialize<Mtx>({{-86.40, 153.30, -108.90, 8.60, -61.60},
-                                    {7.70, -77.00, 3.30, -149.20, 74.80},
-                                    {-121.40, 37.10, 55.30, -74.20, -19.20},
-                                    {-111.40, -22.60, 110.10, -106.20, 88.90},
-                                    {-0.70, 111.70, 154.40, 235.00, -76.50}},
-                                   exec))
+                  .on(exec))
     {
         auto small_size = gko::dim<2>{3, 2};
         small_b = gko::initialize<Mtx>(
@@ -238,7 +234,7 @@ TYPED_TEST(Gcr, SolvesStencilSystemMixed)
     solver->apply(b.get(), x.get());
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 3.0, 2.0}),
-                        (r_mixed<value_type, TypeParam>()));
+                        (r_mixed<value_type, TypeParam>() * 1e1));
 }
 
 
@@ -260,7 +256,7 @@ TYPED_TEST(Gcr, SolvesStencilSystemComplex)
     GKO_ASSERT_MTX_NEAR(x,
                         l({value_type{1.0, -2.0}, value_type{3.0, -6.0},
                            value_type{2.0, -4.0}}),
-                        r<value_type>::value * 1e1);
+                        r<value_type>::value);
 }
 
 
@@ -334,7 +330,7 @@ TYPED_TEST(Gcr, SolvesStencilSystemUsingAdvancedApplyMixed)
     solver->apply(alpha.get(), b.get(), beta.get(), x.get());
 
     GKO_ASSERT_MTX_NEAR(x, l({1.5, 5.0, 2.0}),
-                        (r_mixed<value_type, TypeParam>()) * 1e1);
+                        (r_mixed<value_type, TypeParam>() * 2e1));
 }
 
 
@@ -413,6 +409,8 @@ TYPED_TEST(Gcr, SolvesBigDenseSystem1)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver = this->gcr_factory_big->generate(this->mtx_big);
     auto b = gko::initialize<Mtx>(
         {72748.36, 297469.88, 347229.24, 36290.66, 82958.82, -80192.15},
@@ -430,6 +428,8 @@ TYPED_TEST(Gcr, SolvesBigDenseSystem2)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver = this->gcr_factory_big->generate(this->mtx_big);
     auto b = gko::initialize<Mtx>(
         {175352.10, 313410.50, 131114.10, -134116.30, 179529.30, -43564.90},
@@ -447,6 +447,8 @@ TYPED_TEST(Gcr, SolveWithImplicitResNormCritIsDisabled)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver = this->gcr_factory_big2->generate(this->mtx_big);
     auto b = gko::initialize<Mtx>(
         {175352.10, 313410.50, 131114.10, -134116.30, 179529.30, -43564.90},
@@ -460,7 +462,7 @@ TYPED_TEST(Gcr, SolveWithImplicitResNormCritIsDisabled)
 template <typename T>
 gko::remove_complex<T> infNorm(gko::matrix::Dense<T>* mat, size_t col = 0)
 {
-    using std::abs;
+    using gko::abs;
     using no_cpx_t = gko::remove_complex<T>;
     no_cpx_t norm = 0.0;
     for (size_t i = 0; i < mat->get_size()[0]; ++i) {
@@ -475,6 +477,8 @@ TYPED_TEST(Gcr, SolvesMultipleDenseSystemForDivergenceCheck)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver = this->gcr_factory_big->generate(this->mtx_big);
     auto b1 = gko::initialize<Mtx>(
         {1300083.0, 1018120.5, 906410.0, -42679.5, 846779.5, 1176858.5},
@@ -541,6 +545,8 @@ TYPED_TEST(Gcr, SolvesBigDenseSystem1WithRestart)
     using Mtx = typename TestFixture::Mtx;
     using Solver = typename TestFixture::Solver;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto half_tol = std::sqrt(r<value_type>::value);
     auto gcr_factory_restart =
         Solver::build()
@@ -566,6 +572,8 @@ TYPED_TEST(Gcr, SolvesWithPreconditioner)
     using Mtx = typename TestFixture::Mtx;
     using Solver = typename TestFixture::Solver;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto gcr_factory_preconditioner =
         Solver::build()
             .with_criteria(gko::stop::Iteration::build().with_max_iters(100u),
@@ -592,6 +600,8 @@ TYPED_TEST(Gcr, SolvesTransposedBigDenseSystem)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver = this->gcr_factory_big->generate(this->mtx_big->transpose());
     auto b = gko::initialize<Mtx>(
         {72748.36, 297469.88, 347229.24, 36290.66, 82958.82, -80192.15},
@@ -609,6 +619,8 @@ TYPED_TEST(Gcr, SolvesConjTransposedBigDenseSystem)
 {
     using Mtx = typename TestFixture::Mtx;
     using value_type = typename TestFixture::value_type;
+    // the system is already out of half precision range
+    SKIP_IF_HALF(value_type);
     auto solver =
         this->gcr_factory_big->generate(this->mtx_big->conj_transpose());
     auto b = gko::initialize<Mtx>(
