@@ -4,20 +4,16 @@
 
 #include "core/components/fill_array_kernels.hpp"
 
-
 #include <memory>
 #include <random>
 #include <vector>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/array.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 template <typename T>
@@ -40,14 +36,14 @@ protected:
     gko::array<value_type> seqs;
 };
 
-TYPED_TEST_SUITE(FillArray, gko::test::ValueAndIndexTypes,
+TYPED_TEST_SUITE(FillArray, gko::test::ComplexAndPODTypes,
                  TypenameNameGenerator);
 
 
 TYPED_TEST(FillArray, EqualsReference)
 {
     using T = typename TestFixture::value_type;
-    gko::kernels::EXEC_NAMESPACE::components::fill_array(
+    gko::kernels::GKO_DEVICE_NAMESPACE::components::fill_array(
         this->exec, this->dvals.get_data(), this->total_size, T(1523));
 
     GKO_ASSERT_ARRAY_EQ(this->vals, this->dvals);
@@ -57,7 +53,7 @@ TYPED_TEST(FillArray, EqualsReference)
 TYPED_TEST(FillArray, FillSeqEqualsReference)
 {
     using T = typename TestFixture::value_type;
-    gko::kernels::EXEC_NAMESPACE::components::fill_seq_array(
+    gko::kernels::GKO_DEVICE_NAMESPACE::components::fill_seq_array(
         this->exec, this->dvals.get_data(), this->total_size);
 
     GKO_ASSERT_ARRAY_EQ(this->seqs, this->dvals);
