@@ -1,16 +1,13 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/matrix/coo_kernels.hpp"
 
-
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/executor.hpp>
@@ -19,10 +16,9 @@
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
 
-
 #include "core/test/utils.hpp"
 #include "core/test/utils/unsort_matrix.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Coo : public CommonTestFixture {
@@ -351,3 +347,27 @@ TEST_F(Coo, OutplaceAbsoluteMatrixIsEquivalentToRef)
 
     GKO_ASSERT_MTX_NEAR(abs_mtx, dabs_mtx, r<value_type>::value);
 }
+
+
+TEST_F(Coo, TransposeIsEquivalentToRef)
+{
+    set_up_apply_data();
+
+    auto trans = gko::as<Mtx>(mtx->transpose());
+    auto dtrans = gko::as<Mtx>(dmtx->transpose());
+
+    GKO_ASSERT_MTX_EQ_SPARSITY(dtrans, trans);
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+}
+
+
+TEST_F(Coo, ConjugateTransposeIsEquivalentToRef)
+{
+    set_up_apply_data();
+
+    auto trans = gko::as<Mtx>(mtx->conj_transpose());
+    auto dtrans = gko::as<Mtx>(dmtx->conj_transpose());
+
+    GKO_ASSERT_MTX_EQ_SPARSITY(dtrans, trans);
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+}
