@@ -1,20 +1,16 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/core/matrix/csr.hpp>
-
-
 #include <random>
 #include <stdexcept>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/coo.hpp>
+#include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
 #include <ginkgo/core/matrix/ell.hpp>
@@ -25,21 +21,19 @@
 #include <ginkgo/core/matrix/sellp.hpp>
 #include <ginkgo/core/matrix/sparsity_csr.hpp>
 
-
 #include "core/components/prefix_sum_kernels.hpp"
 #include "core/matrix/csr_kernels.hpp"
 #include "core/test/utils.hpp"
 #include "core/test/utils/assertions.hpp"
 #include "core/test/utils/unsort_matrix.hpp"
 #include "core/utils/matrix_utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Csr : public CommonTestFixture {
 protected:
-    using Arr = gko::array<int>;
+    using Arr = gko::array<index_type>;
     using Vec = gko::matrix::Dense<value_type>;
-    using Vec2 = gko::matrix::Dense<gko::next_precision<value_type>>;
     using Mtx = gko::matrix::Csr<value_type>;
     using ComplexVec = gko::matrix::Dense<std::complex<value_type>>;
     using ComplexMtx = gko::matrix::Csr<std::complex<value_type>>;
@@ -128,27 +122,17 @@ protected:
         square_mtx = Mtx::create(ref, strategy);
         square_mtx->move_from(gen_mtx<Vec>(mtx_size[0], mtx_size[0], 1));
         expected = gen_mtx<Vec>(mtx_size[0], num_vectors, 1);
-        expected2 = Vec2::create(ref);
-        expected2->copy_from(expected);
         y = gen_mtx<Vec>(mtx_size[1], num_vectors, 1);
-        y2 = Vec2::create(ref);
-        y2->copy_from(y);
         alpha = gko::initialize<Vec>({2.0}, ref);
-        alpha2 = gko::initialize<Vec2>({2.0}, ref);
         beta = gko::initialize<Vec>({-1.0}, ref);
-        beta2 = gko::initialize<Vec2>({-1.0}, ref);
         dmtx = Mtx::create(exec, strategy);
         dmtx->copy_from(mtx);
         dsquare_mtx = Mtx::create(exec, strategy);
         dsquare_mtx->copy_from(square_mtx);
         dresult = gko::clone(exec, expected);
-        dresult2 = gko::clone(exec, expected2);
         dy = gko::clone(exec, y);
-        dy2 = gko::clone(exec, y2);
         dalpha = gko::clone(exec, alpha);
-        dalpha2 = gko::clone(exec, alpha2);
         dbeta = gko::clone(exec, beta);
-        dbeta2 = gko::clone(exec, beta2);
 
         std::vector<int> tmp(mtx->get_size()[0], 0);
         auto rng = std::default_random_engine{};
@@ -201,26 +185,18 @@ protected:
     std::unique_ptr<ComplexMtx> complex_mtx;
     std::unique_ptr<Mtx> square_mtx;
     std::unique_ptr<Vec> expected;
-    std::unique_ptr<Vec2> expected2;
     std::unique_ptr<Vec> y;
-    std::unique_ptr<Vec2> y2;
     std::unique_ptr<Vec> alpha;
-    std::unique_ptr<Vec2> alpha2;
     std::unique_ptr<Vec> beta;
-    std::unique_ptr<Vec2> beta2;
 
     std::unique_ptr<Mtx> dmtx;
     std::unique_ptr<Mtx> dmtx2;
     std::unique_ptr<ComplexMtx> dcomplex_mtx;
     std::unique_ptr<Mtx> dsquare_mtx;
     std::unique_ptr<Vec> dresult;
-    std::unique_ptr<Vec2> dresult2;
     std::unique_ptr<Vec> dy;
-    std::unique_ptr<Vec2> dy2;
     std::unique_ptr<Vec> dalpha;
-    std::unique_ptr<Vec2> dalpha2;
     std::unique_ptr<Vec> dbeta;
-    std::unique_ptr<Vec2> dbeta2;
     std::unique_ptr<Arr> rpermute_idxs;
     std::unique_ptr<Arr> cpermute_idxs;
     std::unique_ptr<Perm> rpermutation;
@@ -502,10 +478,10 @@ TEST_F(Csr, AdvancedApplyToCsrMatrixIsEquivalentToRef)
 {
     set_up_apply_data<Mtx::classical>();
     auto trans = mtx->transpose();
-    auto d_trans = dmtx->transpose();
+    auto dtrans = dmtx->transpose();
 
     mtx->apply(alpha, trans, beta, square_mtx);
-    dmtx->apply(dalpha, d_trans, dbeta, dsquare_mtx);
+    dmtx->apply(dalpha, dtrans, dbeta, dsquare_mtx);
 
     GKO_ASSERT_MTX_NEAR(dsquare_mtx, square_mtx, r<value_type>::value);
     GKO_ASSERT_MTX_EQ_SPARSITY(dsquare_mtx, square_mtx);
@@ -517,10 +493,10 @@ TEST_F(Csr, SimpleApplyToCsrMatrixIsEquivalentToRef)
 {
     set_up_apply_data<Mtx::classical>();
     auto trans = mtx->transpose();
-    auto d_trans = dmtx->transpose();
+    auto dtrans = dmtx->transpose();
 
     mtx->apply(trans, square_mtx);
-    dmtx->apply(d_trans, dsquare_mtx);
+    dmtx->apply(dtrans, dsquare_mtx);
 
     GKO_ASSERT_MTX_NEAR(dsquare_mtx, square_mtx, r<value_type>::value);
     GKO_ASSERT_MTX_EQ_SPARSITY(dsquare_mtx, square_mtx);
@@ -659,10 +635,10 @@ TEST_F(Csr, TransposeIsEquivalentToRef)
     set_up_apply_data<Mtx::classical>();
 
     auto trans = gko::as<Mtx>(mtx->transpose());
-    auto d_trans = gko::as<Mtx>(dmtx->transpose());
+    auto dtrans = gko::as<Mtx>(dmtx->transpose());
 
-    GKO_ASSERT_MTX_NEAR(d_trans, trans, 0.0);
-    ASSERT_TRUE(d_trans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
 }
 
 
@@ -673,10 +649,56 @@ TEST_F(Csr, Transpose64IsEquivalentToRef)
     auto dmtx = gko::clone(exec, mtx);
 
     auto trans = gko::as<Mtx64>(mtx->transpose());
-    auto d_trans = gko::as<Mtx64>(dmtx->transpose());
+    auto dtrans = gko::as<Mtx64>(dmtx->transpose());
 
-    GKO_ASSERT_MTX_NEAR(d_trans, trans, 0.0);
-    ASSERT_TRUE(d_trans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
+}
+
+
+TEST_F(Csr, TransposeReuseIsEquivalentToRef)
+{
+    set_up_apply_data<Mtx::classical>();
+
+    auto [trans, reuse] = mtx->transpose_reuse();
+    auto [dtrans, dreuse] = dmtx->transpose_reuse();
+
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_EQ_SPARSITY(dreuse.value_permutation,
+                               reuse.value_permutation);
+    // test that the value permutation works: modify input values
+    mtx->create_value_view()->scale(alpha);
+    dmtx->create_value_view()->scale(dalpha);
+    reuse.update_values(mtx, trans);
+    dreuse.update_values(dmtx, dtrans);
+    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(mtx->transpose()), trans, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Mtx>(dmtx->transpose()), dtrans, 0);
+}
+
+
+TEST_F(Csr, TransposeReuse64IsEquivalentToRef)
+{
+    SKIP_IF_SINGLE_MODE;
+    using Mtx64 = gko::matrix::Csr<value_type, gko::int64>;
+    set_up_apply_data<Mtx::classical>();
+    auto mtx = gen_mtx<Mtx64>(123, 234, 0);
+    auto dmtx = gko::clone(exec, mtx);
+
+    auto [trans, reuse] = mtx->transpose_reuse();
+    auto [dtrans, dreuse] = dmtx->transpose_reuse();
+
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_EQ_SPARSITY(dreuse.value_permutation,
+                               reuse.value_permutation);
+    // test that the value permutation works: modify input values
+    mtx->create_value_view()->scale(alpha);
+    dmtx->create_value_view()->scale(dalpha);
+    reuse.update_values(mtx, trans);
+    dreuse.update_values(dmtx, dtrans);
+    GKO_ASSERT_MTX_NEAR(gko::as<Mtx64>(mtx->transpose()), trans, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Mtx64>(dmtx->transpose()), dtrans, 0);
 }
 
 
@@ -685,10 +707,10 @@ TEST_F(Csr, ConjugateTransposeIsEquivalentToRef)
     set_up_apply_complex_data<ComplexMtx::classical>();
 
     auto trans = gko::as<ComplexMtx>(complex_mtx->conj_transpose());
-    auto d_trans = gko::as<ComplexMtx>(dcomplex_mtx->conj_transpose());
+    auto dtrans = gko::as<ComplexMtx>(dcomplex_mtx->conj_transpose());
 
-    GKO_ASSERT_MTX_NEAR(d_trans, trans, 0.0);
-    ASSERT_TRUE(d_trans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
 }
 
 
@@ -699,10 +721,10 @@ TEST_F(Csr, ConjugateTranspose64IsEquivalentToRef)
     auto dmtx = gko::clone(exec, mtx);
 
     auto trans = gko::as<Mtx64>(mtx->transpose());
-    auto d_trans = gko::as<Mtx64>(dmtx->transpose());
+    auto dtrans = gko::as<Mtx64>(dmtx->transpose());
 
-    GKO_ASSERT_MTX_NEAR(d_trans, trans, 0.0);
-    ASSERT_TRUE(d_trans->is_sorted_by_column_index());
+    GKO_ASSERT_MTX_NEAR(dtrans, trans, 0.0);
+    ASSERT_TRUE(dtrans->is_sorted_by_column_index());
 }
 
 
@@ -900,6 +922,38 @@ TEST_F(Csr, IsGenericPermutable)
 }
 
 
+TEST_F(Csr, IsGenericReusePermutable)
+{
+    using gko::matrix::permute_mode;
+    set_up_apply_data<Mtx::classical>();
+
+    for (auto mode :
+         {permute_mode::none, permute_mode::rows, permute_mode::columns,
+          permute_mode::symmetric, permute_mode::inverse_rows,
+          permute_mode::inverse_columns, permute_mode::inverse_symmetric}) {
+        SCOPED_TRACE(mode);
+        auto [permuted, reuse] = square_mtx->permute_reuse(rpermutation, mode);
+        auto [dpermuted, dreuse] =
+            dsquare_mtx->permute_reuse(rpermutation, mode);
+
+        GKO_ASSERT_MTX_NEAR(permuted, dpermuted, 0);
+        GKO_ASSERT_MTX_EQ_SPARSITY(permuted, dpermuted);
+        ASSERT_TRUE(dpermuted->is_sorted_by_column_index());
+        GKO_ASSERT_MTX_EQ_SPARSITY(reuse.value_permutation,
+                                   dreuse.value_permutation);
+        // test that the value permutation works: modify input values
+        square_mtx->create_value_view()->scale(alpha);
+        dsquare_mtx->create_value_view()->scale(dalpha);
+        reuse.update_values(square_mtx, permuted);
+        dreuse.update_values(dsquare_mtx, dpermuted);
+        GKO_ASSERT_MTX_NEAR(square_mtx->permute(rpermutation, mode), permuted,
+                            0);
+        GKO_ASSERT_MTX_NEAR(dsquare_mtx->permute(rpermutation, mode), dpermuted,
+                            0);
+    }
+}
+
+
 TEST_F(Csr, IsColPermutableHypersparse)
 {
     using gko::matrix::permute_mode;
@@ -945,7 +999,6 @@ TEST_F(Csr, IsGenericPermutableRectangular)
 
 TEST_F(Csr, IsNonsymmPermutable)
 {
-    using gko::matrix::permute_mode;
     set_up_apply_data<Mtx::classical>();
 
     for (auto invert : {false, true}) {
@@ -960,6 +1013,36 @@ TEST_F(Csr, IsNonsymmPermutable)
 }
 
 
+TEST_F(Csr, IsNonsymmReusePermutable)
+{
+    using gko::matrix::permute_mode;
+    set_up_apply_data<Mtx::classical>();
+
+    for (auto invert : {false, true}) {
+        SCOPED_TRACE(invert);
+        auto [permuted, reuse] =
+            mtx->permute_reuse(rpermutation, cpermutation, invert);
+        auto [dpermuted, dreuse] =
+            dmtx->permute_reuse(rpermutation, cpermutation, invert);
+
+        GKO_ASSERT_MTX_NEAR(permuted, dpermuted, 0);
+        GKO_ASSERT_MTX_EQ_SPARSITY(permuted, dpermuted);
+        ASSERT_TRUE(dpermuted->is_sorted_by_column_index());
+        GKO_ASSERT_MTX_EQ_SPARSITY(reuse.value_permutation,
+                                   dreuse.value_permutation);
+        // test that the value permutation works: modify input values
+        mtx->create_value_view()->scale(alpha);
+        dmtx->create_value_view()->scale(dalpha);
+        reuse.update_values(mtx, permuted);
+        dreuse.update_values(dmtx, dpermuted);
+        GKO_ASSERT_MTX_NEAR(mtx->permute(rpermutation, cpermutation, invert),
+                            permuted, 0);
+        GKO_ASSERT_MTX_NEAR(dmtx->permute(rpermutation, cpermutation, invert),
+                            dpermuted, 0);
+    }
+}
+
+
 TEST_F(Csr, IsGenericScalePermutable)
 {
     using gko::matrix::permute_mode;
@@ -1051,6 +1134,7 @@ TEST_F(Csr, IsPermutable)
     auto permuted = gko::as<Mtx>(square_mtx->permute(rpermute_idxs.get()));
     auto dpermuted = gko::as<Mtx>(dsquare_mtx->permute(rpermute_idxs.get()));
 
+    ASSERT_TRUE(dpermuted->is_sorted_by_column_index());
     GKO_ASSERT_MTX_EQ_SPARSITY(permuted, dpermuted);
     GKO_ASSERT_MTX_NEAR(permuted, dpermuted, 0);
 }
@@ -1065,6 +1149,7 @@ TEST_F(Csr, IsInversePermutable)
     auto dpermuted =
         gko::as<Mtx>(dsquare_mtx->inverse_permute(rpermute_idxs.get()));
 
+    ASSERT_TRUE(dpermuted->is_sorted_by_column_index());
     GKO_ASSERT_MTX_EQ_SPARSITY(permuted, dpermuted);
     GKO_ASSERT_MTX_NEAR(permuted, dpermuted, 0);
 }
@@ -1077,6 +1162,7 @@ TEST_F(Csr, IsRowPermutable)
     auto r_permute = gko::as<Mtx>(mtx->row_permute(rpermute_idxs.get()));
     auto dr_permute = gko::as<Mtx>(dmtx->row_permute(rpermute_idxs.get()));
 
+    ASSERT_TRUE(dr_permute->is_sorted_by_column_index());
     GKO_ASSERT_MTX_EQ_SPARSITY(r_permute, dr_permute);
     GKO_ASSERT_MTX_NEAR(r_permute, dr_permute, 0);
 }
@@ -1104,6 +1190,7 @@ TEST_F(Csr, IsInverseRowPermutable)
     auto d_inverse_r_permute =
         gko::as<Mtx>(dmtx->inverse_row_permute(rpermute_idxs.get()));
 
+    ASSERT_TRUE(d_inverse_r_permute->is_sorted_by_column_index());
     GKO_ASSERT_MTX_EQ_SPARSITY(inverse_r_permute, d_inverse_r_permute);
     GKO_ASSERT_MTX_NEAR(inverse_r_permute, d_inverse_r_permute, 0);
 }
@@ -1346,7 +1433,7 @@ TEST_F(Csr, CalculateNnzPerRowInSpanIsEquivalentToRef)
 
     gko::kernels::reference::csr::calculate_nonzeros_per_row_in_span(
         this->ref, this->mtx2.get(), rspan, cspan, &row_nnz);
-    gko::kernels::EXEC_NAMESPACE::csr::calculate_nonzeros_per_row_in_span(
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::calculate_nonzeros_per_row_in_span(
         this->exec, this->dmtx2.get(), rspan, cspan, &drow_nnz);
 
     GKO_ASSERT_ARRAY_EQ(row_nnz, drow_nnz);
@@ -1382,7 +1469,7 @@ TEST_F(Csr, ComputeSubmatrixIsEquivalentToRef)
 
     gko::kernels::reference::csr::compute_submatrix(this->ref, this->mtx2.get(),
                                                     rspan, cspan, smat1.get());
-    gko::kernels::EXEC_NAMESPACE::csr::compute_submatrix(
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::compute_submatrix(
         this->exec, this->dmtx2.get(), rspan, cspan, sdmat1.get());
 
     GKO_ASSERT_MTX_NEAR(sdmat1, smat1, 0.0);
@@ -1408,8 +1495,9 @@ TEST_F(Csr, CalculateNnzPerRowInIndexSetIsEquivalentToRef)
 
     gko::kernels::reference::csr::calculate_nonzeros_per_row_in_index_set(
         this->ref, this->mtx2.get(), rset, cset, row_nnz.get_data());
-    gko::kernels::EXEC_NAMESPACE::csr::calculate_nonzeros_per_row_in_index_set(
-        this->exec, this->dmtx2.get(), drset, dcset, drow_nnz.get_data());
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::
+        calculate_nonzeros_per_row_in_index_set(
+            this->exec, this->dmtx2.get(), drset, dcset, drow_nnz.get_data());
 
     GKO_ASSERT_ARRAY_EQ(row_nnz, drow_nnz);
 }
@@ -1446,7 +1534,7 @@ TEST_F(Csr, ComputeSubmatrixFromIndexSetIsEquivalentToRef)
 
     gko::kernels::reference::csr::compute_submatrix_from_index_set(
         this->ref, this->mtx2.get(), rset, cset, smat1.get());
-    gko::kernels::EXEC_NAMESPACE::csr::compute_submatrix_from_index_set(
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::compute_submatrix_from_index_set(
         this->exec, this->dmtx2.get(), drset, dcset, sdmat1.get());
 
     GKO_ASSERT_MTX_NEAR(sdmat1, smat1, 0.0);
@@ -1501,7 +1589,7 @@ TEST_F(Csr, CanDetectMissingDiagonalEntry)
     auto mtx = gko::clone(exec, ref_mtx);
     bool has_diags = true;
 
-    gko::kernels::EXEC_NAMESPACE::csr::check_diagonal_entries_exist(
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::check_diagonal_entries_exist(
         exec, mtx.get(), has_diags);
 
     ASSERT_FALSE(has_diags);
@@ -1516,7 +1604,7 @@ TEST_F(Csr, CanDetectWhenAllDiagonalEntriesArePresent)
     auto mtx = gko::clone(exec, ref_mtx);
     bool has_diags = true;
 
-    gko::kernels::EXEC_NAMESPACE::csr::check_diagonal_entries_exist(
+    gko::kernels::GKO_DEVICE_NAMESPACE::csr::check_diagonal_entries_exist(
         exec, mtx.get(), has_diags);
 
     ASSERT_TRUE(has_diags);
