@@ -1,24 +1,20 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/matrix/fbcsr_kernels.hpp"
 
-
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/fbcsr.hpp>
 
-
 #include "core/test/matrix/fbcsr_sample.hpp"
 #include "core/test/utils.hpp"
 #include "core/test/utils/fb_matrix_generator.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 template <typename T>
@@ -41,7 +37,7 @@ protected:
 
     std::unique_ptr<const Mtx> rsorted;
 
-    std::normal_distribution<gko::remove_complex<T>> distb;
+    std::normal_distribution<> distb;
     std::default_random_engine engine;
 
     value_type get_random_value()
@@ -52,19 +48,20 @@ protected:
     void generate_sin(gko::ptr_param<Dense> x)
     {
         value_type* const xarr = x->get_values();
+        // we do not have sin for half, so we compute sin in double or
+        // complex<double>
+        using working_type = std::conditional_t<gko::is_complex<value_type>(),
+                                                std::complex<double>, double>;
         for (index_type i = 0; i < x->get_size()[0] * x->get_size()[1]; i++) {
-            xarr[i] =
-                static_cast<real_type>(2.0) *
-                std::sin(static_cast<real_type>(i / 2.0) + get_random_value());
+            xarr[i] = static_cast<value_type>(
+                2.0 * std::sin(i / 2.0 +
+                               static_cast<working_type>(get_random_value())));
         }
     }
 };
 
-#ifdef GKO_COMPILING_HIP
-TYPED_TEST_SUITE(Fbcsr, gko::test::RealValueTypes, TypenameNameGenerator);
-#else
 TYPED_TEST_SUITE(Fbcsr, gko::test::ValueTypes, TypenameNameGenerator);
-#endif
+
 
 TYPED_TEST(Fbcsr, CanWriteFromMatrixOnDevice)
 {
@@ -127,6 +124,12 @@ TYPED_TEST(Fbcsr, SpmvIsEquivalentToRefSorted)
     using Mtx = typename TestFixture::Mtx;
     using Dense = typename TestFixture::Dense;
     using value_type = typename Mtx::value_type;
+    if (this->exec->get_master() != this->exec) {
+        // FBCSR on accelerator does not have half/bfloat16 precision apply
+        // through vendor libraries.
+        SKIP_IF_HALF(value_type);
+        SKIP_IF_BFLOAT16(value_type);
+    }
     auto drand = gko::clone(this->exec, this->rsorted);
     auto x =
         Dense::create(this->ref, gko::dim<2>(this->rsorted->get_size()[1], 1));
@@ -149,6 +152,12 @@ TYPED_TEST(Fbcsr, SpmvMultiIsEquivalentToRefSorted)
     using Mtx = typename TestFixture::Mtx;
     using Dense = typename TestFixture::Dense;
     using value_type = typename Mtx::value_type;
+    if (this->exec->get_master() != this->exec) {
+        // FBCSR on accelerator does not have half/bfloat16 precision apply
+        // through vendor libraries.
+        SKIP_IF_HALF(value_type);
+        SKIP_IF_BFLOAT16(value_type);
+    }
     auto drand = gko::clone(this->exec, this->rsorted);
     auto x =
         Dense::create(this->ref, gko::dim<2>(this->rsorted->get_size()[1], 3));
@@ -172,6 +181,12 @@ TYPED_TEST(Fbcsr, AdvancedSpmvIsEquivalentToRefSorted)
     using Dense = typename TestFixture::Dense;
     using value_type = typename TestFixture::value_type;
     using real_type = typename TestFixture::real_type;
+    if (this->exec->get_master() != this->exec) {
+        // FBCSR on accelerator does not have half/bfloat16 precision apply
+        // through vendor libraries.
+        SKIP_IF_HALF(value_type);
+        SKIP_IF_BFLOAT16(value_type);
+    }
     auto drand = gko::clone(this->exec, this->rsorted);
     auto x =
         Dense::create(this->ref, gko::dim<2>(this->rsorted->get_size()[1], 1));
@@ -202,6 +217,12 @@ TYPED_TEST(Fbcsr, AdvancedSpmvMultiIsEquivalentToRefSorted)
     using Dense = typename TestFixture::Dense;
     using value_type = typename TestFixture::value_type;
     using real_type = typename TestFixture::real_type;
+    if (this->exec->get_master() != this->exec) {
+        // FBCSR on accelerator does not have half/bfloat16 precision apply
+        // through vendor libraries.
+        SKIP_IF_HALF(value_type);
+        SKIP_IF_BFLOAT16(value_type);
+    }
     auto drand = gko::clone(this->exec, this->rsorted);
     auto x =
         Dense::create(this->ref, gko::dim<2>(this->rsorted->get_size()[1], 3));
