@@ -5,16 +5,13 @@
 #include <algorithm>
 #include <numeric>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/matrix/dense.hpp>
 #include <ginkgo/core/matrix/permutation.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Permutation : public CommonTestFixture {
