@@ -5,15 +5,12 @@
 #include <algorithm>
 #include <numeric>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/matrix/scaled_permutation.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class ScaledPermutation : public CommonTestFixture {
