@@ -1,45 +1,14 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <memory>
 #include <random>
 
-
 #include <mpi.h>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/config.hpp>
 #include <ginkgo/core/base/array.hpp>
 #include <ginkgo/core/base/matrix_data.hpp>
@@ -47,9 +16,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/distributed/vector.hpp>
 #include <ginkgo/core/log/logger.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/mpi/executor.hpp"
+#include "test/utils/mpi/common_fixture.hpp"
 
 
 bool needs_transfers(std::shared_ptr<const gko::Executor> exec)
@@ -475,8 +443,8 @@ public:
         dense_real_res = real_dense_type ::create(exec);
         real_res = real_dense_type ::create(exec);
 
-        dense_tmp = gko::Array<char>(exec);
-        tmp = gko::Array<char>(exec);
+        dense_tmp = gko::array<char>(exec);
+        tmp = gko::array<char>(exec);
 
         auto num_parts =
             static_cast<gko::experimental::distributed::comm_index_type>(
@@ -494,8 +462,7 @@ public:
                                                            global_index_type>(
             size[0], size[1],
             std::uniform_int_distribution<gko::size_type>(size[1], size[1]),
-            std::normal_distribution<gko::remove_complex<value_type>>(),
-            engine);
+            std::normal_distribution<>(), engine);
         dense_x->read(md_x);
         auto tmp_x = dist_vec_type::create(ref, comm);
         tmp_x->read_distributed(md_x, part);
@@ -505,8 +472,7 @@ public:
                                                            global_index_type>(
             size[0], size[1],
             std::uniform_int_distribution<gko::size_type>(size[1], size[1]),
-            std::normal_distribution<gko::remove_complex<value_type>>(),
-            engine);
+            std::normal_distribution<>(), engine);
         dense_y->read(md_y);
         auto tmp_y = dist_vec_type::create(ref, comm);
         tmp_y->read_distributed(md_y, part);
@@ -676,6 +642,30 @@ TYPED_TEST(VectorReductions, ComputeSquaredNorm2WithTmpIsSameAsDense)
 }
 
 
+TYPED_TEST(VectorReductions, ComputesMeanIsSameAsDense)
+{
+    using value_type = typename TestFixture::value_type;
+    this->init_result();
+
+    this->x->compute_mean(this->res);
+    this->dense_x->compute_mean(this->dense_res);
+
+    GKO_ASSERT_MTX_NEAR(this->res, this->dense_res, r<value_type>::value);
+}
+
+
+TYPED_TEST(VectorReductions, ComputesMeanWithTmpIsSameAsDense)
+{
+    using value_type = typename TestFixture::value_type;
+    this->init_result();
+
+    this->x->compute_mean(this->res, this->tmp);
+    this->dense_x->compute_mean(this->dense_res, this->dense_tmp);
+
+    GKO_ASSERT_MTX_NEAR(this->res, this->dense_res, r<value_type>::value);
+}
+
+
 TYPED_TEST(VectorReductions, ComputeDotCopiesToHostOnlyIfNecessary)
 {
     this->init_result();
@@ -770,8 +760,7 @@ public:
             local_size[0], local_size[1],
             std::uniform_int_distribution<gko::size_type>(local_size[1],
                                                           local_size[1]),
-            std::normal_distribution<gko::remove_complex<vtype>>(), engine,
-            exec);
+            std::normal_distribution<>(), engine, exec);
         dist = DistVectorType::create(exec, comm, size, gko::clone(local));
     }
 
@@ -783,8 +772,7 @@ public:
         alpha = gko::test::generate_random_matrix<dense_type>(
             1, size[1],
             std::uniform_int_distribution<gko::size_type>(size[1], size[1]),
-            std::normal_distribution<gko::remove_complex<value_type>>(), engine,
-            exec);
+            std::normal_distribution<>(), engine, exec);
     }
 
     void init_complex_vectors()
@@ -978,8 +966,7 @@ TYPED_TEST(VectorLocalOps, FillSameAsLocal)
 {
     using value_type = typename TestFixture::value_type;
     auto value = gko::test::detail::get_rand_value<value_type>(
-        std::normal_distribution<gko::remove_complex<value_type>>(),
-        this->engine);
+        std::normal_distribution<>(), this->engine);
     this->init_vectors();
 
     this->x->fill(value);
