@@ -4,13 +4,11 @@
 
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/composition.hpp>
 #include <ginkgo/core/reorder/mc64.hpp>
 
-
 #include "core/test/utils/assertions.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 namespace {
