@@ -4,27 +4,26 @@
 
 #include "core/solver/batch_bicgstab_kernels.hpp"
 
-
 #include <memory>
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/batch_multi_vector.hpp>
 #include <ginkgo/core/log/batch_logger.hpp>
 #include <ginkgo/core/matrix/batch_csr.hpp>
 #include <ginkgo/core/matrix/batch_dense.hpp>
 #include <ginkgo/core/matrix/batch_ell.hpp>
+#include <ginkgo/core/matrix/batch_identity.hpp>
+#include <ginkgo/core/preconditioner/batch_jacobi.hpp>
 #include <ginkgo/core/solver/batch_bicgstab.hpp>
 
-
 #include "core/base/batch_utilities.hpp"
+#include "core/base/dispatch_helper.hpp"
 #include "core/matrix/batch_dense_kernels.hpp"
 #include "core/test/utils.hpp"
 #include "core/test/utils/batch_helpers.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class BatchBicgstab : public CommonTestFixture {
@@ -52,9 +51,13 @@ protected:
                                   const gko::batch::BatchLinOp* prec,
                                   const Mtx* mtx, const MVec* b, MVec* x,
                                   LogData& log_data) {
-            gko::kernels::EXEC_NAMESPACE::batch_bicgstab::apply<
-                typename Mtx::value_type>(executor, settings, mtx, prec, b, x,
-                                          log_data);
+            gko::run<gko::batch::matrix::Identity<value_type>,
+                     gko::batch::preconditioner::Jacobi<value_type>>(
+                prec, [&](auto preconditioner) {
+                    gko::kernels::GKO_DEVICE_NAMESPACE::batch_bicgstab::apply(
+                        executor, settings, mtx, preconditioner, b, x,
+                        log_data);
+                });
         };
         solver_settings = Settings{max_iters, tol,
                                    gko::batch::stop::tolerance_type::relative};
