@@ -4,15 +4,12 @@
 
 #include "core/solver/cb_gmres_kernels.hpp"
 
-
 #include <algorithm>
 #include <cmath>
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
@@ -21,10 +18,9 @@
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 
-
 #include "core/solver/cb_gmres_accessor.hpp"
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class CbGmres : public CommonTestFixture {
@@ -150,7 +146,7 @@ protected:
         auto& krylov_bases = range_helper.get_bases();
         d_to_host = d_range_helper.get_bases();
         const auto tolerance = r<storage_type>::value;
-        using std::abs;
+        using gko::abs;
         for (gko::size_type i = 0; i < krylov_bases.get_size(); ++i) {
             const auto ref_value = krylov_bases.get_const_data()[i];
             const auto dev_value = d_to_host.get_const_data()[i];
@@ -209,7 +205,7 @@ TEST_F(CbGmres, CbGmresInitialize1IsEquivalentToRef)
     gko::kernels::reference::cb_gmres::initialize(
         ref, b.get(), residual.get(), givens_sin.get(), givens_cos.get(),
         stop_status.get(), default_krylov_dim_mixed);
-    gko::kernels::EXEC_NAMESPACE::cb_gmres::initialize(
+    gko::kernels::GKO_DEVICE_NAMESPACE::cb_gmres::initialize(
         exec, d_b.get(), d_residual.get(), d_givens_sin.get(),
         d_givens_cos.get(), d_stop_status.get(), default_krylov_dim_mixed);
 
@@ -230,7 +226,7 @@ TEST_F(CbGmres, CbGmresInitialize2IsEquivalentToRef)
         residual_norm_collection.get(), arnoldi_norm.get(),
         range_helper.get_range(), next_krylov_basis.get(),
         final_iter_nums.get(), tmp, default_krylov_dim_mixed);
-    gko::kernels::EXEC_NAMESPACE::cb_gmres::restart(
+    gko::kernels::GKO_DEVICE_NAMESPACE::cb_gmres::restart(
         exec, d_residual.get(), d_residual_norm.get(),
         d_residual_norm_collection.get(), d_arnoldi_norm.get(),
         d_range_helper.get_range(), d_next_krylov_basis.get(),
@@ -255,7 +251,7 @@ TEST_F(CbGmres, CbGmresStep1IsEquivalentToRef)
         range_helper.get_range(), hessenberg_iter.get(), buffer_iter.get(),
         arnoldi_norm.get(), iter, final_iter_nums.get(), stop_status.get(),
         reorth_status.get(), num_reorth.get());
-    gko::kernels::EXEC_NAMESPACE::cb_gmres::arnoldi(
+    gko::kernels::GKO_DEVICE_NAMESPACE::cb_gmres::arnoldi(
         exec, d_next_krylov_basis.get(), d_givens_sin.get(), d_givens_cos.get(),
         d_residual_norm.get(), d_residual_norm_collection.get(),
         d_range_helper.get_range(), d_hessenberg_iter.get(),
@@ -285,7 +281,7 @@ TEST_F(CbGmres, CbGmresStep2IsEquivalentToRef)
         ref, residual_norm_collection.get(),
         range_helper.get_range().get_accessor().to_const(), hessenberg.get(),
         y.get(), before_preconditioner.get(), final_iter_nums.get());
-    gko::kernels::EXEC_NAMESPACE::cb_gmres::solve_krylov(
+    gko::kernels::GKO_DEVICE_NAMESPACE::cb_gmres::solve_krylov(
         exec, d_residual_norm_collection.get(),
         d_range_helper.get_range().get_accessor().to_const(),
         d_hessenberg.get(), d_y.get(), d_before_preconditioner.get(),
