@@ -4,13 +4,10 @@
 
 #include "core/solver/fcg_kernels.hpp"
 
-
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
@@ -19,10 +16,9 @@
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 
-
 #include "core/test/utils.hpp"
 #include "core/utils/matrix_utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Fcg : public CommonTestFixture {
@@ -122,7 +118,7 @@ TEST_F(Fcg, FcgInitializeIsEquivalentToRef)
     gko::kernels::reference::fcg::initialize(
         ref, b.get(), r.get(), z.get(), p.get(), q.get(), t.get(),
         prev_rho.get(), rho.get(), rho_t.get(), stop_status.get());
-    gko::kernels::EXEC_NAMESPACE::fcg::initialize(
+    gko::kernels::GKO_DEVICE_NAMESPACE::fcg::initialize(
         exec, d_b.get(), d_r.get(), d_z.get(), d_p.get(), d_q.get(), d_t.get(),
         d_prev_rho.get(), d_rho.get(), d_rho_t.get(), d_stop_status.get());
 
@@ -144,9 +140,9 @@ TEST_F(Fcg, FcgStep1IsEquivalentToRef)
 
     gko::kernels::reference::fcg::step_1(ref, p.get(), z.get(), rho_t.get(),
                                          prev_rho.get(), stop_status.get());
-    gko::kernels::EXEC_NAMESPACE::fcg::step_1(exec, d_p.get(), d_z.get(),
-                                              d_rho_t.get(), d_prev_rho.get(),
-                                              d_stop_status.get());
+    gko::kernels::GKO_DEVICE_NAMESPACE::fcg::step_1(
+        exec, d_p.get(), d_z.get(), d_rho_t.get(), d_prev_rho.get(),
+        d_stop_status.get());
 
     GKO_ASSERT_MTX_NEAR(d_p, p, ::r<value_type>::value);
     GKO_ASSERT_MTX_NEAR(d_z, z, ::r<value_type>::value);
@@ -159,7 +155,7 @@ TEST_F(Fcg, FcgStep2IsEquivalentToRef)
     gko::kernels::reference::fcg::step_2(ref, x.get(), r.get(), t.get(),
                                          p.get(), q.get(), beta.get(),
                                          rho.get(), stop_status.get());
-    gko::kernels::EXEC_NAMESPACE::fcg::step_2(
+    gko::kernels::GKO_DEVICE_NAMESPACE::fcg::step_2(
         exec, d_x.get(), d_r.get(), d_t.get(), d_p.get(), d_q.get(),
         d_beta.get(), d_rho.get(), d_stop_status.get());
 
