@@ -4,13 +4,10 @@
 
 #include "core/solver/ir_kernels.hpp"
 
-
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
@@ -19,9 +16,8 @@
 #include <ginkgo/core/stop/combined.hpp>
 #include <ginkgo/core/stop/iteration.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Ir : public CommonTestFixture {
@@ -55,7 +51,7 @@ TEST_F(Ir, InitializeIsEquivalentToRef)
     auto d_stop_status = gko::array<gko::stopping_status>(exec, stop_status);
 
     gko::kernels::reference::ir::initialize(ref, &stop_status);
-    gko::kernels::EXEC_NAMESPACE::ir::initialize(exec, &d_stop_status);
+    gko::kernels::GKO_DEVICE_NAMESPACE::ir::initialize(exec, &d_stop_status);
 
     auto tmp = gko::array<gko::stopping_status>(ref, d_stop_status);
     for (int i = 0; i < stop_status.get_size(); ++i) {
