@@ -4,13 +4,10 @@
 
 #include "core/solver/multigrid_kernels.hpp"
 
-
 #include <random>
 
-
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/base/exception.hpp>
 #include <ginkgo/core/base/executor.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
@@ -18,9 +15,8 @@
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
 
-
 #include "core/test/utils.hpp"
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 class Multigrid : public CommonTestFixture {
@@ -144,7 +140,7 @@ TEST_F(Multigrid, MultigridKCycleStep1IsEquivalentToRef)
 
     gko::kernels::reference::multigrid::kcycle_step_1(
         ref, alpha.get(), rho.get(), v.get(), g.get(), d.get(), e.get());
-    gko::kernels::EXEC_NAMESPACE::multigrid::kcycle_step_1(
+    gko::kernels::GKO_DEVICE_NAMESPACE::multigrid::kcycle_step_1(
         exec, d_alpha.get(), d_rho.get(), d_v.get(), d_g.get(), d_d.get(),
         d_e.get());
 
@@ -161,7 +157,7 @@ TEST_F(Multigrid, MultigridKCycleStep2IsEquivalentToRef)
     gko::kernels::reference::multigrid::kcycle_step_2(
         ref, alpha.get(), rho.get(), gamma.get(), beta.get(), zeta.get(),
         d.get(), e.get());
-    gko::kernels::EXEC_NAMESPACE::multigrid::kcycle_step_2(
+    gko::kernels::GKO_DEVICE_NAMESPACE::multigrid::kcycle_step_2(
         exec, d_alpha.get(), d_rho.get(), d_gamma.get(), d_beta.get(),
         d_zeta.get(), d_d.get(), d_e.get());
 
@@ -179,11 +175,11 @@ TEST_F(Multigrid, MultigridKCycleCheckStopIsEquivalentToRef)
 
     gko::kernels::reference::multigrid::kcycle_check_stop(
         ref, old_norm.get(), new_norm.get(), 1.0, is_stop_10);
-    gko::kernels::EXEC_NAMESPACE::multigrid::kcycle_check_stop(
+    gko::kernels::GKO_DEVICE_NAMESPACE::multigrid::kcycle_check_stop(
         exec, d_old_norm.get(), d_new_norm.get(), 1.0, d_is_stop_10);
     gko::kernels::reference::multigrid::kcycle_check_stop(
         ref, old_norm.get(), new_norm.get(), 0.5, is_stop_5);
-    gko::kernels::EXEC_NAMESPACE::multigrid::kcycle_check_stop(
+    gko::kernels::GKO_DEVICE_NAMESPACE::multigrid::kcycle_check_stop(
         exec, d_old_norm.get(), d_new_norm.get(), 0.5, d_is_stop_5);
 
     GKO_ASSERT_EQ(d_is_stop_10, is_stop_10);
