@@ -4,12 +4,10 @@
 
 #include <gtest/gtest.h>
 
-
 #include <ginkgo/core/stop/criterion.hpp>
 #include <ginkgo/core/stop/iteration.hpp>
 
-
-#include "test/utils/executor.hpp"
+#include "test/utils/common_fixture.hpp"
 
 
 constexpr gko::size_type test_iterations = 10;
