@@ -1,10 +1,7 @@
-// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+// SPDX-FileCopyrightText: 2017 - 2025 The Ginkgo authors
 //
 // SPDX-License-Identifier: BSD-3-Clause
 
-#include <ginkgo/ginkgo.hpp>
-
-
 #include <chrono>
 #include <cmath>
 #include <cstdlib>
@@ -17,6 +14,8 @@
 #include <utility>
 #include <vector>
 
+#include <ginkgo/ginkgo.hpp>
+
 
 void assert_similar_matrices(gko::ptr_param<const gko::matrix::Dense<>> m1,
                              gko::ptr_param<const gko::matrix::Dense<>> m2,
@@ -463,12 +462,28 @@ int main()
         check_solver<Solver>(exec, A_raw, b, x);
     }
 
+    // core/solver/chebyshev.hpp
+    {
+        using Solver = gko::solver::Chebyshev<>;
+        auto test =
+            Solver::build()
+                .with_criteria(
+                    gko::stop::Iteration::build().with_max_iters(1u).on(exec))
+                .on(exec);
+    }
+
     // core/solver/fcg.hpp
     {
         using Solver = gko::solver::Fcg<>;
         check_solver<Solver>(exec, A_raw, b, x);
     }
 
+    // core/solver/pipe_cg.hpp
+    {
+        using Solver = gko::solver::PipeCg<>;
+        check_solver<Solver>(exec, A_raw, b, x);
+    }
+
     // core/solver/gcr.hpp
     {
         using Solver = gko::solver::Gcr<>;
